// This file has been generated by Py++.

#ifndef PropertyDefinitionColour_hpp__pyplusplus_wrapper
#define PropertyDefinitionColour_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColour_class();

#endif//PropertyDefinitionColour_hpp__pyplusplus_wrapper
