// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColourUVector2_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColourUVector2_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColourUVector2_class();

#endif//FalagardPropertyBaseColourUVector2_hpp__pyplusplus_wrapper
