// This file has been generated by Py++.

#ifndef ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556_hpp__pyplusplus_wrapper
#define ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556_class();

#endif//ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556_hpp__pyplusplus_wrapper
