// This file has been generated by Py++.

#ifndef ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae_hpp__pyplusplus_wrapper
#define ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae_class();

#endif//ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae_hpp__pyplusplus_wrapper
