// This file has been generated by Py++.

#ifndef SchemeIterator_hpp__pyplusplus_wrapper
#define SchemeIterator_hpp__pyplusplus_wrapper

void register_SchemeIterator_class();

#endif//SchemeIterator_hpp__pyplusplus_wrapper
