// This file has been generated by Py++.

#ifndef FontGlyph_hpp__pyplusplus_wrapper
#define FontGlyph_hpp__pyplusplus_wrapper

void register_FontGlyph_class();

#endif//FontGlyph_hpp__pyplusplus_wrapper
