// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "GUIContextRenderTargetEventArgs.pypp.hpp"

namespace bp = boost::python;

struct GUIContextRenderTargetEventArgs_wrapper : CEGUI::GUIContextRenderTargetEventArgs, bp::wrapper< CEGUI::GUIContextRenderTargetEventArgs > {

    GUIContextRenderTargetEventArgs_wrapper(CEGUI::GUIContextRenderTargetEventArgs const & arg )
    : CEGUI::GUIContextRenderTargetEventArgs( arg )
      , bp::wrapper< CEGUI::GUIContextRenderTargetEventArgs >(){
        // copy constructor
        
    }

    GUIContextRenderTargetEventArgs_wrapper(::CEGUI::GUIContext * context, ::CEGUI::RenderTarget * target )
    : CEGUI::GUIContextRenderTargetEventArgs( boost::python::ptr(context), boost::python::ptr(target) )
      , bp::wrapper< CEGUI::GUIContextRenderTargetEventArgs >(){
        // constructor
    
    }

    static ::CEGUI::RenderTarget * get_renderTarget(CEGUI::GUIContextRenderTargetEventArgs const & inst ){
        return inst.renderTarget;
    }
    
    static void set_renderTarget( CEGUI::GUIContextRenderTargetEventArgs & inst, ::CEGUI::RenderTarget * new_value ){ 
        inst.renderTarget = new_value;
    }

};

void register_GUIContextRenderTargetEventArgs_class(){

    { //::CEGUI::GUIContextRenderTargetEventArgs
        typedef bp::class_< GUIContextRenderTargetEventArgs_wrapper, bp::bases< CEGUI::GUIContextEventArgs > > GUIContextRenderTargetEventArgs_exposer_t;
        GUIContextRenderTargetEventArgs_exposer_t GUIContextRenderTargetEventArgs_exposer = GUIContextRenderTargetEventArgs_exposer_t( "GUIContextRenderTargetEventArgs", "! EventArgs class passed for GUIContext RenderTarget related events.\n", bp::init< CEGUI::GUIContext *, CEGUI::RenderTarget * >(( bp::arg("context"), bp::arg("target") )) );
        bp::scope GUIContextRenderTargetEventArgs_scope( GUIContextRenderTargetEventArgs_exposer );
        GUIContextRenderTargetEventArgs_exposer.add_property( "renderTarget"
                    , bp::make_function( (::CEGUI::RenderTarget * (*)( ::CEGUI::GUIContextRenderTargetEventArgs const & ))(&GUIContextRenderTargetEventArgs_wrapper::get_renderTarget), bp::return_internal_reference< >() )
                    , bp::make_function( (void (*)( ::CEGUI::GUIContextRenderTargetEventArgs &,::CEGUI::RenderTarget * ))(&GUIContextRenderTargetEventArgs_wrapper::set_renderTarget), bp::with_custodian_and_ward_postcall< 1, 2 >() ) );
    }

}
