// This file has been generated by Py++.

#ifndef TypedPropertyColour_hpp__pyplusplus_wrapper
#define TypedPropertyColour_hpp__pyplusplus_wrapper

void register_TypedPropertyColour_class();

#endif//TypedPropertyColour_hpp__pyplusplus_wrapper
