// This file has been generated by Py++.

#ifndef TypedPropertyFloat_hpp__pyplusplus_wrapper
#define TypedPropertyFloat_hpp__pyplusplus_wrapper

void register_TypedPropertyFloat_class();

#endif//TypedPropertyFloat_hpp__pyplusplus_wrapper
