// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionVerticalTextFormatting_class();

#endif//PropertyLinkDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper
