// This file has been generated by Py++.

#ifndef RenderEffectManager_hpp__pyplusplus_wrapper
#define RenderEffectManager_hpp__pyplusplus_wrapper

void register_RenderEffectManager_class();

#endif//RenderEffectManager_hpp__pyplusplus_wrapper
