// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "KeyEventArgs.pypp.hpp"

namespace bp = boost::python;

void register_KeyEventArgs_class(){

    { //::CEGUI::KeyEventArgs
        typedef bp::class_< CEGUI::KeyEventArgs, bp::bases< CEGUI::WindowEventArgs > > KeyEventArgs_exposer_t;
        KeyEventArgs_exposer_t KeyEventArgs_exposer = KeyEventArgs_exposer_t( "KeyEventArgs", "*!\n\
        \n\
           EventArgs based class that is used for objects passed to input event handlers\n\
           concerning keyboard input.\n\
        *\n", bp::init< CEGUI::Window * >(( bp::arg("wnd") )) );
        bp::scope KeyEventArgs_scope( KeyEventArgs_exposer );
        bp::implicitly_convertible< CEGUI::Window *, CEGUI::KeyEventArgs >();
        KeyEventArgs_exposer.def_readwrite( "codepoint", &CEGUI::KeyEventArgs::codepoint );
        KeyEventArgs_exposer.def_readwrite( "scancode", &CEGUI::KeyEventArgs::scancode );
        KeyEventArgs_exposer.def_readwrite( "sysKeys", &CEGUI::KeyEventArgs::sysKeys );
    }

}
