// This file has been generated by Py++.

#ifndef PropertyDefinitionColourRect_hpp__pyplusplus_wrapper
#define PropertyDefinitionColourRect_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColourRect_class();

#endif//PropertyDefinitionColourRect_hpp__pyplusplus_wrapper
