// This file has been generated by Py++.

#ifndef SingletonWindowManager_hpp__pyplusplus_wrapper
#define SingletonWindowManager_hpp__pyplusplus_wrapper

void register_SingletonWindowManager_class();

#endif//SingletonWindowManager_hpp__pyplusplus_wrapper
