// This file has been generated by Py++.

#ifndef MultiLineEditbox_hpp__pyplusplus_wrapper
#define MultiLineEditbox_hpp__pyplusplus_wrapper

void register_MultiLineEditbox_class();

#endif//MultiLineEditbox_hpp__pyplusplus_wrapper
