// This file has been generated by Py++.

#ifndef PropertyDefinitionColourURect_hpp__pyplusplus_wrapper
#define PropertyDefinitionColourURect_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColourURect_class();

#endif//PropertyDefinitionColourURect_hpp__pyplusplus_wrapper
