// This file has been generated by Py++.

#ifndef OgreRenderer_hpp__pyplusplus_wrapper
#define OgreRenderer_hpp__pyplusplus_wrapper

void register_OgreRenderer_class();

#endif//OgreRenderer_hpp__pyplusplus_wrapper
