// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionSizef_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionSizef_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionSizef_class();

#endif//PropertyLinkDefinitionSizef_hpp__pyplusplus_wrapper
