// This file has been generated by Py++.

#ifndef LayoutContainer_hpp__pyplusplus_wrapper
#define LayoutContainer_hpp__pyplusplus_wrapper

void register_LayoutContainer_class();

#endif//LayoutContainer_hpp__pyplusplus_wrapper
