// This file has been generated by Py++.

#ifndef ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364_hpp__pyplusplus_wrapper
#define ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364_class();

#endif//ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364_hpp__pyplusplus_wrapper
