// This file has been generated by Py++.

#ifndef ItemListbox_hpp__pyplusplus_wrapper
#define ItemListbox_hpp__pyplusplus_wrapper

void register_ItemListbox_class();

#endif//ItemListbox_hpp__pyplusplus_wrapper
