// This file has been generated by Py++.

#ifndef StateImageryMap_hpp__pyplusplus_wrapper
#define StateImageryMap_hpp__pyplusplus_wrapper

void register_StateImageryMap_class();

#endif//StateImageryMap_hpp__pyplusplus_wrapper
