// This file has been generated by Py++.

#ifndef FalagardPropertyBaseSizef_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseSizef_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseSizef_class();

#endif//FalagardPropertyBaseSizef_hpp__pyplusplus_wrapper
