// This file has been generated by Py++.

#ifndef WindowManager_hpp__pyplusplus_wrapper
#define WindowManager_hpp__pyplusplus_wrapper

void register_WindowManager_class();

#endif//WindowManager_hpp__pyplusplus_wrapper
