// This file has been generated by Py++.

#ifndef TypedPropertyBool_hpp__pyplusplus_wrapper
#define TypedPropertyBool_hpp__pyplusplus_wrapper

void register_TypedPropertyBool_class();

#endif//TypedPropertyBool_hpp__pyplusplus_wrapper
