// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionVerticalFormatting_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionVerticalFormatting_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionVerticalFormatting_class();

#endif//PropertyLinkDefinitionVerticalFormatting_hpp__pyplusplus_wrapper
