// This file has been generated by Py++.

#ifndef MultiColumnList_hpp__pyplusplus_wrapper
#define MultiColumnList_hpp__pyplusplus_wrapper

void register_MultiColumnList_class();

#endif//MultiColumnList_hpp__pyplusplus_wrapper
