// This file has been generated by Py++.

#ifndef PropertyDim_hpp__pyplusplus_wrapper
#define PropertyDim_hpp__pyplusplus_wrapper

void register_PropertyDim_class();

#endif//PropertyDim_hpp__pyplusplus_wrapper
