// This file has been generated by Py++.

#ifndef ConstBaseIterator_ccf46e19ae28e4742a13388873833720_hpp__pyplusplus_wrapper
#define ConstBaseIterator_ccf46e19ae28e4742a13388873833720_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_ccf46e19ae28e4742a13388873833720_class();

#endif//ConstBaseIterator_ccf46e19ae28e4742a13388873833720_hpp__pyplusplus_wrapper
