// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionImage_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionImage_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionImage_class();

#endif//PropertyLinkDefinitionImage_hpp__pyplusplus_wrapper
