// This file has been generated by Py++.

#ifndef NamedXMLResourceManagerScheme_hpp__pyplusplus_wrapper
#define NamedXMLResourceManagerScheme_hpp__pyplusplus_wrapper

void register_NamedXMLResourceManagerScheme_class();

#endif//NamedXMLResourceManagerScheme_hpp__pyplusplus_wrapper
