// This file has been generated by Py++.

#ifndef WindowRenderer_hpp__pyplusplus_wrapper
#define WindowRenderer_hpp__pyplusplus_wrapper

void register_WindowRenderer_class();

#endif//WindowRenderer_hpp__pyplusplus_wrapper
