// This file has been generated by Py++.

#ifndef AnimationEventArgs_hpp__pyplusplus_wrapper
#define AnimationEventArgs_hpp__pyplusplus_wrapper

void register_AnimationEventArgs_class();

#endif//AnimationEventArgs_hpp__pyplusplus_wrapper
