// This file has been generated by Py++.

#include "boost/python.hpp"

#include "indexing_suite/value_traits.hpp"

#include "indexing_suite/container_suite.hpp"

#include "indexing_suite/vector.hpp"

#include "indexing_suite/set.hpp"

#include "indexing_suite/map.hpp"

#include "generators/include/python_CEGUI.h"

#include "AbsoluteDim.pypp.hpp"

#include "ActivationEventArgs.pypp.hpp"

#include "Affector.pypp.hpp"

#include "Animation.pypp.hpp"

#include "AnimationEventArgs.pypp.hpp"

#include "AnimationInstance.pypp.hpp"

#include "AnimationManager.pypp.hpp"

#include "AnimationNameIterator.pypp.hpp"

#include "BaseDim.pypp.hpp"

#include "BasicImage.pypp.hpp"

#include "BasicRenderedStringParser.pypp.hpp"

#include "BoundSlot.pypp.hpp"

#include "ButtonBase.pypp.hpp"

#include "CentredRenderedString.pypp.hpp"

#include "Clipboard.pypp.hpp"

#include "Colour.pypp.hpp"

#include "ColourRect.pypp.hpp"

#include "ComboDropList.pypp.hpp"

#include "Combobox.pypp.hpp"

#include "ComponentArea.pypp.hpp"

#include "ComponentList.pypp.hpp"

#include "Connection.pypp.hpp"

#include "ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133.pypp.hpp"

#include "ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f.pypp.hpp"

#include "ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923.pypp.hpp"

#include "ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c.pypp.hpp"

#include "ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472.pypp.hpp"

#include "ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b.pypp.hpp"

#include "ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb.pypp.hpp"

#include "ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1.pypp.hpp"

#include "ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d.pypp.hpp"

#include "ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174.pypp.hpp"

#include "ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec.pypp.hpp"

#include "ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9.pypp.hpp"

#include "ConstBaseIterator_7f0012c4899354432f095ef9bb577046.pypp.hpp"

#include "ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6.pypp.hpp"

#include "ConstBaseIterator_937e880515ad590c17d4b2b82a347310.pypp.hpp"

#include "ConstBaseIterator_99d0247943ea008baba14993d63dd301.pypp.hpp"

#include "ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda.pypp.hpp"

#include "ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b.pypp.hpp"

#include "ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b.pypp.hpp"

#include "ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364.pypp.hpp"

#include "ConstBaseIterator_ccf46e19ae28e4742a13388873833720.pypp.hpp"

#include "ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd.pypp.hpp"

#include "ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f.pypp.hpp"

#include "ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556.pypp.hpp"

#include "ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae.pypp.hpp"

#include "ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533.pypp.hpp"

#include "CoordConverter.pypp.hpp"

#include "DefaultLogger.pypp.hpp"

#include "DefaultResourceProvider.pypp.hpp"

#include "DefaultWindow.pypp.hpp"

#include "Dimension.pypp.hpp"

#include "DisplayEventArgs.pypp.hpp"

#include "DragContainer.pypp.hpp"

#include "DragDropEventArgs.pypp.hpp"

#include "Editbox.pypp.hpp"

#include "Element.pypp.hpp"

#include "ElementEventArgs.pypp.hpp"

#include "Event.pypp.hpp"

#include "EventAction.pypp.hpp"

#include "EventActionIterator.pypp.hpp"

#include "EventArgs.pypp.hpp"

#include "EventIterator.pypp.hpp"

#include "EventLinkDefinition.pypp.hpp"

#include "EventLinkDefinitionIterator.pypp.hpp"

#include "EventLinkDefinitionMap.pypp.hpp"

#include "EventSet.pypp.hpp"

#include "Exception.pypp.hpp"

#include "FalagardComponentBase.pypp.hpp"

#include "FalagardMappingIterator.pypp.hpp"

#include "FalagardPropertyBaseBool.pypp.hpp"

#include "FalagardPropertyBaseColour.pypp.hpp"

#include "FalagardPropertyBaseColourRect.pypp.hpp"

#include "FalagardPropertyBaseColourUBox.pypp.hpp"

#include "FalagardPropertyBaseColourUDim.pypp.hpp"

#include "FalagardPropertyBaseColourURect.pypp.hpp"

#include "FalagardPropertyBaseColourUVector2.pypp.hpp"

#include "FalagardPropertyBaseFloat.pypp.hpp"

#include "FalagardPropertyBaseFont.pypp.hpp"

#include "FalagardPropertyBaseHorizontalFormatting.pypp.hpp"

#include "FalagardPropertyBaseHorizontalTextFormatting.pypp.hpp"

#include "FalagardPropertyBaseImage.pypp.hpp"

#include "FalagardPropertyBaseRectf.pypp.hpp"

#include "FalagardPropertyBaseSizef.pypp.hpp"

#include "FalagardPropertyBaseString.pypp.hpp"

#include "FalagardPropertyBaseUint.pypp.hpp"

#include "FalagardPropertyBaseVector2f.pypp.hpp"

#include "FalagardPropertyBaseVerticalFormatting.pypp.hpp"

#include "FalagardPropertyBaseVerticalTextFormatting.pypp.hpp"

#include "FalagardXMLHelper.pypp.hpp"

#include "Font.pypp.hpp"

#include "FontDim.pypp.hpp"

#include "FontEventArgs.pypp.hpp"

#include "FontGlyph.pypp.hpp"

#include "FontIterator.pypp.hpp"

#include "FontManager.pypp.hpp"

#include "FrameComponent.pypp.hpp"

#include "FrameComponentIterator.pypp.hpp"

#include "FrameComponentVector.pypp.hpp"

#include "FrameWindow.pypp.hpp"

#include "GUIContext.pypp.hpp"

#include "GUIContextEventArgs.pypp.hpp"

#include "GUIContextRenderTargetEventArgs.pypp.hpp"

#include "GeometryBuffer.pypp.hpp"

#include "GlobalEventSet.pypp.hpp"

#include "GridLayoutContainer.pypp.hpp"

#include "GroupBox.pypp.hpp"

#include "HeaderSequenceEventArgs.pypp.hpp"

#include "HorizontalLayoutContainer.pypp.hpp"

#include "Image.pypp.hpp"

#include "ImageCodec.pypp.hpp"

#include "ImageDim.pypp.hpp"

#include "ImageIterator.pypp.hpp"

#include "ImageManager.pypp.hpp"

#include "ImagePropertyDim.pypp.hpp"

#include "ImageryComponent.pypp.hpp"

#include "ImageryComponentIterator.pypp.hpp"

#include "ImageryComponentVector.pypp.hpp"

#include "ImageryIterator.pypp.hpp"

#include "ImagerySection.pypp.hpp"

#include "ImagerySectionMap.pypp.hpp"

#include "Interpolator.pypp.hpp"

#include "ItemEntry.pypp.hpp"

#include "ItemListBase.pypp.hpp"

#include "ItemListbox.pypp.hpp"

#include "Key.pypp.hpp"

#include "KeyEventArgs.pypp.hpp"

#include "KeyFrame.pypp.hpp"

#include "LBItemList.pypp.hpp"

#include "LayerIterator.pypp.hpp"

#include "LayerSpecification.pypp.hpp"

#include "LayerSpecificationVector.pypp.hpp"

#include "LayoutContainer.pypp.hpp"

#include "LineList.pypp.hpp"

#include "LinkTargetIterator.pypp.hpp"

#include "LinkedEventArgs.pypp.hpp"

#include "ListHeader.pypp.hpp"

#include "ListHeaderSegment.pypp.hpp"

#include "Listbox.pypp.hpp"

#include "ListboxItem.pypp.hpp"

#include "ListboxTextItem.pypp.hpp"

#include "LoadableUIElementIterator.pypp.hpp"

#include "Logger.pypp.hpp"

#include "MCLGridRef.pypp.hpp"

#include "MenuBase.pypp.hpp"

#include "MenuItem.pypp.hpp"

#include "Menubar.pypp.hpp"

#include "MouseCursor.pypp.hpp"

#include "MouseCursorEventArgs.pypp.hpp"

#include "MouseEventArgs.pypp.hpp"

#include "MultiColumnList.pypp.hpp"

#include "MultiLineEditbox.pypp.hpp"

#include "NamedArea.pypp.hpp"

#include "NamedAreaIterator.pypp.hpp"

#include "NamedAreaMap.pypp.hpp"

#include "NamedElement.pypp.hpp"

#include "NamedElementEventArgs.pypp.hpp"

#include "NamedXMLResourceManagerFont.pypp.hpp"

#include "NamedXMLResourceManagerScheme.pypp.hpp"

#include "OperatorDim.pypp.hpp"

#include "PopupMenu.pypp.hpp"

#include "ProgressBar.pypp.hpp"

#include "Property.pypp.hpp"

#include "PropertyDefinitionBase.pypp.hpp"

#include "PropertyDefinitionBaseMap.pypp.hpp"

#include "PropertyDefinitionBool.pypp.hpp"

#include "PropertyDefinitionColour.pypp.hpp"

#include "PropertyDefinitionColourRect.pypp.hpp"

#include "PropertyDefinitionColourUBox.pypp.hpp"

#include "PropertyDefinitionColourUDim.pypp.hpp"

#include "PropertyDefinitionColourURect.pypp.hpp"

#include "PropertyDefinitionColourUVector2.pypp.hpp"

#include "PropertyDefinitionFloat.pypp.hpp"

#include "PropertyDefinitionFont.pypp.hpp"

#include "PropertyDefinitionHorizontalFormatting.pypp.hpp"

#include "PropertyDefinitionHorizontalTextFormatting.pypp.hpp"

#include "PropertyDefinitionImage.pypp.hpp"

#include "PropertyDefinitionIterator.pypp.hpp"

#include "PropertyDefinitionRectf.pypp.hpp"

#include "PropertyDefinitionSizef.pypp.hpp"

#include "PropertyDefinitionString.pypp.hpp"

#include "PropertyDefinitionUint.pypp.hpp"

#include "PropertyDefinitionVector2f.pypp.hpp"

#include "PropertyDefinitionVerticalFormatting.pypp.hpp"

#include "PropertyDefinitionVerticalTextFormatting.pypp.hpp"

#include "PropertyDim.pypp.hpp"

#include "PropertyHelper.pypp.hpp"

#include "PropertyInitialiser.pypp.hpp"

#include "PropertyInitialiserIterator.pypp.hpp"

#include "PropertyInitialiserMap.pypp.hpp"

#include "PropertyInitialiserVector.pypp.hpp"

#include "PropertyIterator.pypp.hpp"

#include "PropertyLinkDefinitionBool.pypp.hpp"

#include "PropertyLinkDefinitionColour.pypp.hpp"

#include "PropertyLinkDefinitionColourRect.pypp.hpp"

#include "PropertyLinkDefinitionColourUBox.pypp.hpp"

#include "PropertyLinkDefinitionColourUDim.pypp.hpp"

#include "PropertyLinkDefinitionColourURect.pypp.hpp"

#include "PropertyLinkDefinitionColourUVector2.pypp.hpp"

#include "PropertyLinkDefinitionFloat.pypp.hpp"

#include "PropertyLinkDefinitionFont.pypp.hpp"

#include "PropertyLinkDefinitionHorizontalFormatting.pypp.hpp"

#include "PropertyLinkDefinitionHorizontalTextFormatting.pypp.hpp"

#include "PropertyLinkDefinitionImage.pypp.hpp"

#include "PropertyLinkDefinitionRectf.pypp.hpp"

#include "PropertyLinkDefinitionSizef.pypp.hpp"

#include "PropertyLinkDefinitionString.pypp.hpp"

#include "PropertyLinkDefinitionUint.pypp.hpp"

#include "PropertyLinkDefinitionVector2f.pypp.hpp"

#include "PropertyLinkDefinitionVerticalFormatting.pypp.hpp"

#include "PropertyLinkDefinitionVerticalTextFormatting.pypp.hpp"

#include "PropertyReceiver.pypp.hpp"

#include "PropertySet.pypp.hpp"

#include "PushButton.pypp.hpp"

#include "PyCEGUI_enumerations.pypp.hpp"

#include "Quaternion.pypp.hpp"

#include "RadioButton.pypp.hpp"

#include "RawDataContainer.pypp.hpp"

#include "Rectf.pypp.hpp"

#include "RegexMatchStateEventArgs.pypp.hpp"

#include "RegexMatcher.pypp.hpp"

#include "RenderEffect.pypp.hpp"

#include "RenderEffectFactory.pypp.hpp"

#include "RenderEffectManager.pypp.hpp"

#include "RenderQueue.pypp.hpp"

#include "RenderQueueEventArgs.pypp.hpp"

#include "RenderTarget.pypp.hpp"

#include "RenderTargetEventArgs.pypp.hpp"

#include "RenderedString.pypp.hpp"

#include "RenderedStringComponent.pypp.hpp"

#include "RenderedStringImageComponent.pypp.hpp"

#include "RenderedStringParser.pypp.hpp"

#include "RenderedStringTextComponent.pypp.hpp"

#include "RenderedStringWidgetComponent.pypp.hpp"

#include "Renderer.pypp.hpp"

#include "RenderingContext.pypp.hpp"

#include "RenderingSurface.pypp.hpp"

#include "RenderingWindow.pypp.hpp"

#include "ResourceEventArgs.pypp.hpp"

#include "ResourceEventSet.pypp.hpp"

#include "ResourceProvider.pypp.hpp"

#include "Scheme.pypp.hpp"

#include "SchemeIterator.pypp.hpp"

#include "SchemeManager.pypp.hpp"

#include "ScriptModule.pypp.hpp"

#include "ScrollablePane.pypp.hpp"

#include "Scrollbar.pypp.hpp"

#include "ScrolledContainer.pypp.hpp"

#include "ScrolledItemListBase.pypp.hpp"

#include "SectionIterator.pypp.hpp"

#include "SectionSpecification.pypp.hpp"

#include "SectionSpecificationVector.pypp.hpp"

#include "SequentialLayoutContainer.pypp.hpp"

#include "SimpleTimer.pypp.hpp"

#include "SingletonAnimationManager.pypp.hpp"

#include "SingletonFontManager.pypp.hpp"

#include "SingletonGlobalEventSet.pypp.hpp"

#include "SingletonImageManager.pypp.hpp"

#include "SingletonLogger.pypp.hpp"

#include "SingletonRenderEffectManager.pypp.hpp"

#include "SingletonSchemeManager.pypp.hpp"

#include "SingletonSystem.pypp.hpp"

#include "SingletonWidgetLookManager.pypp.hpp"

#include "SingletonWindowFactoryManager.pypp.hpp"

#include "SingletonWindowManager.pypp.hpp"

#include "SingletonWindowRendererManager.pypp.hpp"

#include "Sizef.pypp.hpp"

#include "Slider.pypp.hpp"

#include "Spinner.pypp.hpp"

#include "StateImagery.pypp.hpp"

#include "StateImageryMap.pypp.hpp"

#include "StateIterator.pypp.hpp"

#include "StdPairCEGUIImageImageFactory.pypp.hpp"

#include "StdPairCEGUIStringString.pypp.hpp"

#include "StdPairFloatFloat.pypp.hpp"

#include "StringSet.pypp.hpp"

#include "Subscriber.pypp.hpp"

#include "System.pypp.hpp"

#include "SystemKeys.pypp.hpp"

#include "TabButton.pypp.hpp"

#include "TabControl.pypp.hpp"

#include "TargetTypeStack.pypp.hpp"

#include "TextComponent.pypp.hpp"

#include "TextComponentIterator.pypp.hpp"

#include "TextComponentVector.pypp.hpp"

#include "TextUtils.pypp.hpp"

#include "Texture.pypp.hpp"

#include "TextureTarget.pypp.hpp"

#include "Thumb.pypp.hpp"

#include "Titlebar.pypp.hpp"

#include "ToggleButton.pypp.hpp"

#include "Tooltip.pypp.hpp"

#include "Tree.pypp.hpp"

#include "TreeEventArgs.pypp.hpp"

#include "TreeItem.pypp.hpp"

#include "TypeAliasIterator.pypp.hpp"

#include "TypedPropertyBool.pypp.hpp"

#include "TypedPropertyColour.pypp.hpp"

#include "TypedPropertyColourRect.pypp.hpp"

#include "TypedPropertyColourUBox.pypp.hpp"

#include "TypedPropertyColourUDim.pypp.hpp"

#include "TypedPropertyColourURect.pypp.hpp"

#include "TypedPropertyColourUVector2.pypp.hpp"

#include "TypedPropertyFloat.pypp.hpp"

#include "TypedPropertyFont.pypp.hpp"

#include "TypedPropertyHorizontalFormatting.pypp.hpp"

#include "TypedPropertyHorizontalTextFormatting.pypp.hpp"

#include "TypedPropertyImage.pypp.hpp"

#include "TypedPropertyRectf.pypp.hpp"

#include "TypedPropertySizef.pypp.hpp"

#include "TypedPropertyString.pypp.hpp"

#include "TypedPropertyUint.pypp.hpp"

#include "TypedPropertyVector2f.pypp.hpp"

#include "TypedPropertyVerticalFormatting.pypp.hpp"

#include "TypedPropertyVerticalTextFormatting.pypp.hpp"

#include "UBox.pypp.hpp"

#include "UDim.pypp.hpp"

#include "URect.pypp.hpp"

#include "USize.pypp.hpp"

#include "UVector2.pypp.hpp"

#include "UnifiedDim.pypp.hpp"

#include "UpdateEventArgs.pypp.hpp"

#include "Vector2f.pypp.hpp"

#include "Vector3f.pypp.hpp"

#include "Vertex.pypp.hpp"

#include "VerticalLayoutContainer.pypp.hpp"

#include "WidgetComponent.pypp.hpp"

#include "WidgetComponentIterator.pypp.hpp"

#include "WidgetComponentMap.pypp.hpp"

#include "WidgetDim.pypp.hpp"

#include "WidgetLookFeel.pypp.hpp"

#include "WidgetLookFeelMap.pypp.hpp"

#include "WidgetLookIterator.pypp.hpp"

#include "WidgetLookManager.pypp.hpp"

#include "Window.pypp.hpp"

#include "WindowEventArgs.pypp.hpp"

#include "WindowFactory.pypp.hpp"

#include "WindowFactoryIterator.pypp.hpp"

#include "WindowFactoryManager.pypp.hpp"

#include "WindowIterator.pypp.hpp"

#include "WindowManager.pypp.hpp"

#include "WindowRenderer.pypp.hpp"

#include "WindowRendererFactory.pypp.hpp"

#include "WindowRendererManager.pypp.hpp"

#include "Workarounds.pypp.hpp"

#include "XMLAttributes.pypp.hpp"

#include "XMLHandler.pypp.hpp"

#include "XMLParser.pypp.hpp"

#include "XMLSerializer.pypp.hpp"

#include "vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater_.pypp.hpp"

#include "vector_less__CEGUI_scope_UDim__greater_.pypp.hpp"

namespace bp = boost::python;

struct CEGUI_String_to_python : public boost::python::converter::expected_from_python_type<CEGUI::String>
{
    static PyObject* convert(const CEGUI::String& s)
    {
        // use native byteorder
        int byteorder = 0;

        // "replace" replaces invalid utf32 chars with "?"
        
        // python wants the size of the buffer, not length of the string,
        // this is the reason for the sizeof
        return boost::python::incref(
            PyUnicode_DecodeUTF32((const char*)(s.ptr()), s.length() * sizeof(CEGUI::utf32), "replace", &byteorder)
        );
    }
};

struct CEGUI_String_from_python
{
    CEGUI_String_from_python()
    {
      boost::python::converter::registry::push_back(
        &convertible,
        &construct,
        boost::python::type_id<CEGUI::String>(),
        &boost::python::converter::expected_from_python_type<CEGUI::String>::get_pytype);
    }
    
    static void* convertible(PyObject* obj_ptr)
    {
        // we allow str() and unicode() objects to be converted to CEGUI::String
        // NOTE: Python3 only has unicode strings
        if (PyUnicode_Check(obj_ptr)
#if PY_MAJOR_VERSION < 3
            || PyString_Check(obj_ptr)
#endif
        )
        {
            return obj_ptr;
        }
        return 0;
    }
    
    static void construct(
      PyObject* obj_ptr,
      boost::python::converter::rvalue_from_python_stage1_data* data)
    {
#if PY_MAJOR_VERSION < 3
        if (!PyUnicode_Check(obj_ptr))
        {
            // no need for unicode, this is just plain str()
            const char* value = PyString_AsString(obj_ptr);
            if (value == 0)
                boost::python::throw_error_already_set();

            void* storage = ((boost::python::converter::rvalue_from_python_storage<CEGUI::String>*)data)->storage.bytes;
            new (storage) CEGUI::String(value);
            data->convertible = storage;
        }
        else
#endif
        {
            // we have to employ a bit more machinery since this is utf16 coded string
            // we encode given unicode object (ucs2 - utf16) to utf8
            PyObject* utf8 = PyUnicode_AsUTF8String(obj_ptr);
            // then we get the raw bytes of the utf8 python object
#if PY_MAJOR_VERSION < 3
            const CEGUI::utf8* value = (CEGUI::utf8*)PyString_AsString(utf8);
#else
            const CEGUI::utf8* value = (CEGUI::utf8*)PyBytes_AsString(utf8);
#endif
            if (value == 0)
                boost::python::throw_error_already_set();

            void* storage = ((boost::python::converter::rvalue_from_python_storage<CEGUI::String>*)data)->storage.bytes;
            new (storage) CEGUI::String(value);
            data->convertible = storage;
            // now we don't need utf8 anymore, CEGUI::String holds the data for us now, by decreasing
            // refcount to utf8, we are practically deallocating it
            boost::python::decref(utf8);
        }
    }
};

BOOST_PYTHON_MODULE(PyCEGUI){
    register_enumerations();

    boost::python::to_python_converter<
        CEGUI::String,
        CEGUI_String_to_python,
        true>();
    
    CEGUI_String_from_python();

    register_vector_less__CEGUI_scope_UDim__greater__class();

    register_LBItemList_class();

    register_TextComponentVector_class();

    register_TargetTypeStack_class();

    register_SectionSpecificationVector_class();

    register_ComponentList_class();

    register_PropertyInitialiserVector_class();

    register_vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__class();

    register_LineList_class();

    register_LayerSpecificationVector_class();

    register_ImageryComponentVector_class();

    register_FrameComponentVector_class();

    register_StringSet_class();

    register_WidgetLookFeelMap_class();

    register_WidgetComponentMap_class();

    register_StateImageryMap_class();

    register_PropertyInitialiserMap_class();

    register_PropertyDefinitionBaseMap_class();

    register_NamedAreaMap_class();

    register_ImagerySectionMap_class();

    register_EventLinkDefinitionMap_class();

    register_BaseDim_class();

    register_AbsoluteDim_class();

    register_EventArgs_class();

    register_WindowEventArgs_class();

    register_ActivationEventArgs_class();

    register_KeyFrame_class();

    register_Affector_class();

    register_Animation_class();

    register_AnimationEventArgs_class();

    register_AnimationInstance_class();

    register_SingletonAnimationManager_class();

    register_AnimationManager_class();

    register_Rectf_class();

    register_Image_class();

    register_BasicImage_class();

    register_RenderedStringParser_class();

    register_BasicRenderedStringParser_class();

    register_BoundSlot_class();

    register_EventSet_class();

    register_PropertyReceiver_class();

    register_PropertySet_class();

    register_Element_class();

    register_NamedElement_class();

    register_Window_class();

    register_ButtonBase_class();

    register_CentredRenderedString_class();

    register_Clipboard_class();

    register_Colour_class();

    bp::implicitly_convertible< CEGUI::Colour, CEGUI::argb_t >();

    register_ColourRect_class();

    register_Listbox_class();

    register_ComboDropList_class();

    register_Combobox_class();

    register_ComponentArea_class();

    register_ConstBaseIterator_7f0012c4899354432f095ef9bb577046_class();

    register_ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec_class();

    register_ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1_class();

    register_ConstBaseIterator_ccf46e19ae28e4742a13388873833720_class();

    register_ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b_class();

    register_ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472_class();

    register_ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda_class();

    register_ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b_class();

    register_ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923_class();

    register_ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174_class();

    register_ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533_class();

    register_ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d_class();

    register_ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f_class();

    register_ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c_class();

    register_ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd_class();

    register_ConstBaseIterator_b33d7bd272dd6c97821cbd9402cb4364_class();

    register_ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f_class();

    register_ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133_class();

    register_ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb_class();

    register_ConstBaseIterator_937e880515ad590c17d4b2b82a347310_class();

    register_ConstBaseIterator_e6837f87c747b0388313d47a6a1e8556_class();

    register_ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9_class();

    register_ConstBaseIterator_99d0247943ea008baba14993d63dd301_class();

    register_ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6_class();

    register_ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b_class();

    register_ConstBaseIterator_e78994bfd2b781a573fdd2cd7ca2ceae_class();

    register_EventIterator_class();

    register_FontIterator_class();

    register_ImageryIterator_class();

    register_NamedAreaIterator_class();

    register_PropertyIterator_class();

    register_SchemeIterator_class();

    register_StateIterator_class();

    register_WidgetLookIterator_class();

    register_WindowFactoryIterator_class();

    register_TypeAliasIterator_class();

    register_FalagardMappingIterator_class();

    register_ImageIterator_class();

    register_EventLinkDefinitionIterator_class();

    register_PropertyDefinitionIterator_class();

    register_PropertyInitialiserIterator_class();

    register_WidgetComponentIterator_class();

    register_LayerIterator_class();

    register_AnimationNameIterator_class();

    register_EventActionIterator_class();

    register_FrameComponentIterator_class();

    register_ImageryComponentIterator_class();

    register_LoadableUIElementIterator_class();

    register_SectionIterator_class();

    register_TextComponentIterator_class();

    register_WindowIterator_class();

    register_LinkTargetIterator_class();

    register_CoordConverter_class();

    register_SingletonLogger_class();

    register_Logger_class();

    register_DefaultLogger_class();

    register_ResourceProvider_class();

    register_DefaultResourceProvider_class();

    register_DefaultWindow_class();

    register_Dimension_class();

    register_DisplayEventArgs_class();

    register_DragContainer_class();

    register_DragDropEventArgs_class();

    register_Editbox_class();

    register_ElementEventArgs_class();

    register_Connection_class();

    register_Event_class();

    register_EventAction_class();

    register_EventLinkDefinition_class();

    register_Exception_class();

    register_FalagardComponentBase_class();

    register_PropertyDefinitionBase_class();

    register_Property_class();

    register_TypedPropertyColour_class();

    register_FalagardPropertyBaseColour_class();

    register_TypedPropertyColourRect_class();

    register_FalagardPropertyBaseColourRect_class();

    register_Font_class();

    register_TypedPropertyFont_class();

    register_FalagardPropertyBaseFont_class();

    register_TypedPropertyHorizontalFormatting_class();

    register_FalagardPropertyBaseHorizontalFormatting_class();

    register_TypedPropertyHorizontalTextFormatting_class();

    register_FalagardPropertyBaseHorizontalTextFormatting_class();

    register_TypedPropertyImage_class();

    register_FalagardPropertyBaseImage_class();

    register_URect_class();

    register_TypedPropertyColourURect_class();

    register_FalagardPropertyBaseColourURect_class();

    register_TypedPropertyRectf_class();

    register_FalagardPropertyBaseRectf_class();

    register_Sizef_class();

    register_TypedPropertySizef_class();

    register_FalagardPropertyBaseSizef_class();

    register_TypedPropertyString_class();

    register_FalagardPropertyBaseString_class();

    register_UBox_class();

    register_TypedPropertyColourUBox_class();

    register_FalagardPropertyBaseColourUBox_class();

    register_UDim_class();

    register_TypedPropertyColourUDim_class();

    register_FalagardPropertyBaseColourUDim_class();

    register_UVector2_class();

    register_TypedPropertyColourUVector2_class();

    register_FalagardPropertyBaseColourUVector2_class();

    register_Vector2f_class();

    register_TypedPropertyVector2f_class();

    register_FalagardPropertyBaseVector2f_class();

    register_TypedPropertyVerticalFormatting_class();

    register_FalagardPropertyBaseVerticalFormatting_class();

    register_TypedPropertyVerticalTextFormatting_class();

    register_FalagardPropertyBaseVerticalTextFormatting_class();

    register_TypedPropertyBool_class();

    register_FalagardPropertyBaseBool_class();

    register_TypedPropertyFloat_class();

    register_FalagardPropertyBaseFloat_class();

    register_TypedPropertyUint_class();

    register_FalagardPropertyBaseUint_class();

    register_FalagardXMLHelper_class();

    register_FontDim_class();

    register_FontEventArgs_class();

    register_FontGlyph_class();

    register_ResourceEventSet_class();

    register_NamedXMLResourceManagerFont_class();

    register_SingletonFontManager_class();

    register_FontManager_class();

    register_FrameComponent_class();

    register_FrameWindow_class();

    register_Key_class();

    register_RenderingSurface_class();

    register_GUIContext_class();

    register_GUIContextEventArgs_class();

    register_GUIContextRenderTargetEventArgs_class();

    register_GeometryBuffer_class();

    register_SingletonGlobalEventSet_class();

    register_GlobalEventSet_class();

    register_LayoutContainer_class();

    register_GridLayoutContainer_class();

    register_GroupBox_class();

    register_HeaderSequenceEventArgs_class();

    register_SequentialLayoutContainer_class();

    register_HorizontalLayoutContainer_class();

    register_ImageCodec_class();

    register_ImageDim_class();

    register_SingletonImageManager_class();

    register_ImageManager_class();

    register_ImagePropertyDim_class();

    register_ImageryComponent_class();

    register_ImagerySection_class();

    register_Interpolator_class();

    register_ItemEntry_class();

    register_ItemListBase_class();

    register_ScrolledItemListBase_class();

    register_ItemListbox_class();

    register_KeyEventArgs_class();

    register_LayerSpecification_class();

    register_LinkedEventArgs_class();

    register_ListHeaderSegment_class();

    register_ListHeader_class();

    register_ListboxItem_class();

    register_ListboxTextItem_class();

    register_MCLGridRef_class();

    register_MenuBase_class();

    register_MenuItem_class();

    register_Menubar_class();

    register_MouseCursor_class();

    register_MouseCursorEventArgs_class();

    register_MouseEventArgs_class();

    register_MultiColumnList_class();

    register_MultiLineEditbox_class();

    register_NamedArea_class();

    register_NamedElementEventArgs_class();

    register_NamedXMLResourceManagerScheme_class();

    register_OperatorDim_class();

    register_PopupMenu_class();

    register_ProgressBar_class();

    register_PropertyDefinitionColour_class();

    register_PropertyDefinitionColourRect_class();

    register_PropertyDefinitionFont_class();

    register_PropertyDefinitionHorizontalFormatting_class();

    register_PropertyDefinitionHorizontalTextFormatting_class();

    register_PropertyDefinitionImage_class();

    register_PropertyDefinitionColourURect_class();

    register_PropertyDefinitionRectf_class();

    register_PropertyDefinitionSizef_class();

    register_PropertyDefinitionString_class();

    register_PropertyDefinitionColourUBox_class();

    register_PropertyDefinitionColourUDim_class();

    register_PropertyDefinitionColourUVector2_class();

    register_PropertyDefinitionVector2f_class();

    register_PropertyDefinitionVerticalFormatting_class();

    register_PropertyDefinitionVerticalTextFormatting_class();

    register_PropertyDefinitionBool_class();

    register_PropertyDefinitionFloat_class();

    register_PropertyDefinitionUint_class();

    register_PropertyDim_class();

    register_PropertyHelper_class();

    register_PropertyInitialiser_class();

    register_PropertyLinkDefinitionColour_class();

    register_PropertyLinkDefinitionColourRect_class();

    register_PropertyLinkDefinitionFont_class();

    register_PropertyLinkDefinitionHorizontalFormatting_class();

    register_PropertyLinkDefinitionHorizontalTextFormatting_class();

    register_PropertyLinkDefinitionImage_class();

    register_PropertyLinkDefinitionColourURect_class();

    register_PropertyLinkDefinitionRectf_class();

    register_PropertyLinkDefinitionSizef_class();

    register_PropertyLinkDefinitionString_class();

    register_PropertyLinkDefinitionColourUBox_class();

    register_PropertyLinkDefinitionColourUDim_class();

    register_PropertyLinkDefinitionColourUVector2_class();

    register_PropertyLinkDefinitionVector2f_class();

    register_PropertyLinkDefinitionVerticalFormatting_class();

    register_PropertyLinkDefinitionVerticalTextFormatting_class();

    register_PropertyLinkDefinitionBool_class();

    register_PropertyLinkDefinitionFloat_class();

    register_PropertyLinkDefinitionUint_class();

    register_PushButton_class();

    register_Quaternion_class();

    register_ToggleButton_class();

    register_RadioButton_class();

    register_RawDataContainer_class();

    register_RegexMatcher_class();

    register_RegexMatchStateEventArgs_class();

    register_RenderEffect_class();

    register_RenderEffectFactory_class();

    register_SingletonRenderEffectManager_class();

    register_RenderEffectManager_class();

    register_RenderQueue_class();

    register_RenderQueueEventArgs_class();

    register_RenderTarget_class();

    register_RenderTargetEventArgs_class();

    register_RenderedString_class();

    register_RenderedStringComponent_class();

    register_RenderedStringImageComponent_class();

    register_RenderedStringTextComponent_class();

    register_RenderedStringWidgetComponent_class();

    register_Renderer_class();

    register_RenderingContext_class();

    register_RenderingWindow_class();

    register_ResourceEventArgs_class();

    register_Scheme_class();

    register_SingletonSchemeManager_class();

    register_SchemeManager_class();

    register_ScriptModule_class();

    register_ScrollablePane_class();

    register_Scrollbar_class();

    register_ScrolledContainer_class();

    register_SectionSpecification_class();

    register_SimpleTimer_class();

    register_SingletonSystem_class();

    register_SingletonWidgetLookManager_class();

    register_SingletonWindowFactoryManager_class();

    register_SingletonWindowManager_class();

    register_SingletonWindowRendererManager_class();

    register_USize_class();

    register_Slider_class();

    register_Spinner_class();

    register_StateImagery_class();

    register_Subscriber_class();

    register_XMLParser_class();

    register_System_class();

    register_SystemKeys_class();

    register_TabButton_class();

    register_TabControl_class();

    register_TextComponent_class();

    register_TextUtils_class();

    register_Texture_class();

    register_TextureTarget_class();

    register_Thumb_class();

    register_Titlebar_class();

    register_Tooltip_class();

    register_Tree_class();

    register_TreeEventArgs_class();

    register_TreeItem_class();

    register_UnifiedDim_class();

    register_UpdateEventArgs_class();

    register_Vector3f_class();

    register_Vertex_class();

    register_VerticalLayoutContainer_class();

    register_WidgetComponent_class();

    register_WidgetDim_class();

    register_WidgetLookFeel_class();

    register_WidgetLookManager_class();

    register_WindowFactory_class();

    register_WindowFactoryManager_class();

    register_WindowManager_class();

    register_WindowRenderer_class();

    register_WindowRendererFactory_class();

    register_WindowRendererManager_class();

    register_Workarounds_class();

    register_XMLAttributes_class();

    register_XMLHandler_class();

    register_XMLSerializer_class();

    bp::implicitly_convertible< CEGUI::XMLSerializer, bool >();

    register_StdPairCEGUIImageImageFactory_class();

    register_StdPairCEGUIStringString_class();

    boost::python::scope().attr("CompileDate__") = __DATE__;

    boost::python::scope().attr("CompileTime__") = __TIME__;

    boost::python::scope().attr("__doc__") = "PyCEGUI - version 0.8.0";

    boost::python::scope().attr("Version__") = "0.8.0";

    register_StdPairFloatFloat_class();
}

