// This file has been generated by Py++.

#ifndef TypedPropertyVector2f_hpp__pyplusplus_wrapper
#define TypedPropertyVector2f_hpp__pyplusplus_wrapper

void register_TypedPropertyVector2f_class();

#endif//TypedPropertyVector2f_hpp__pyplusplus_wrapper
