// This file has been generated by Py++.

#ifndef FalagardXMLHelper_hpp__pyplusplus_wrapper
#define FalagardXMLHelper_hpp__pyplusplus_wrapper

void register_FalagardXMLHelper_class();

#endif//FalagardXMLHelper_hpp__pyplusplus_wrapper
