// This file has been generated by Py++.

#ifndef OpenGL3StateChangeWrapper_hpp__pyplusplus_wrapper
#define OpenGL3StateChangeWrapper_hpp__pyplusplus_wrapper

void register_OpenGL3StateChangeWrapper_class();

#endif//OpenGL3StateChangeWrapper_hpp__pyplusplus_wrapper
