// This file has been generated by Py++.

#ifndef FalagardComponentBase_hpp__pyplusplus_wrapper
#define FalagardComponentBase_hpp__pyplusplus_wrapper

void register_FalagardComponentBase_class();

#endif//FalagardComponentBase_hpp__pyplusplus_wrapper
