// This file has been generated by Py++.

#ifndef ListboxItem_hpp__pyplusplus_wrapper
#define ListboxItem_hpp__pyplusplus_wrapper

void register_ListboxItem_class();

#endif//ListboxItem_hpp__pyplusplus_wrapper
