// This file has been generated by Py++.

#ifndef ScrolledContainer_hpp__pyplusplus_wrapper
#define ScrolledContainer_hpp__pyplusplus_wrapper

void register_ScrolledContainer_class();

#endif//ScrolledContainer_hpp__pyplusplus_wrapper
