// This file has been generated by Py++.

#ifndef ImageryComponentVector_hpp__pyplusplus_wrapper
#define ImageryComponentVector_hpp__pyplusplus_wrapper

void register_ImageryComponentVector_class();

#endif//ImageryComponentVector_hpp__pyplusplus_wrapper
