// This file has been generated by Py++.

#ifndef WindowEventArgs_hpp__pyplusplus_wrapper
#define WindowEventArgs_hpp__pyplusplus_wrapper

void register_WindowEventArgs_class();

#endif//WindowEventArgs_hpp__pyplusplus_wrapper
