// This file has been generated by Py++.

#ifndef ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b_hpp__pyplusplus_wrapper
#define ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b_class();

#endif//ConstBaseIterator_58fce9a3b4ea911d189f971dbf4a6c9b_hpp__pyplusplus_wrapper
