// This file has been generated by Py++.

#ifndef WindowIterator_hpp__pyplusplus_wrapper
#define WindowIterator_hpp__pyplusplus_wrapper

void register_WindowIterator_class();

#endif//WindowIterator_hpp__pyplusplus_wrapper
