// This file has been generated by Py++.

#ifndef ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b_hpp__pyplusplus_wrapper
#define ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b_class();

#endif//ConstBaseIterator_a74f832b0a98ff325f0d295bba46f58b_hpp__pyplusplus_wrapper
