// This file has been generated by Py++.

#ifndef NamedArea_hpp__pyplusplus_wrapper
#define NamedArea_hpp__pyplusplus_wrapper

void register_NamedArea_class();

#endif//NamedArea_hpp__pyplusplus_wrapper
