// This file has been generated by Py++.

#ifndef FalagardPropertyBaseFloat_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseFloat_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseFloat_class();

#endif//FalagardPropertyBaseFloat_hpp__pyplusplus_wrapper
