// This file has been generated by Py++.

#ifndef EventLinkDefinitionMap_hpp__pyplusplus_wrapper
#define EventLinkDefinitionMap_hpp__pyplusplus_wrapper

void register_EventLinkDefinitionMap_class();

#endif//EventLinkDefinitionMap_hpp__pyplusplus_wrapper
