// This file has been generated by Py++.

#ifndef UpdateEventArgs_hpp__pyplusplus_wrapper
#define UpdateEventArgs_hpp__pyplusplus_wrapper

void register_UpdateEventArgs_class();

#endif//UpdateEventArgs_hpp__pyplusplus_wrapper
