// This file has been generated by Py++.

#ifndef RenderedStringComponent_hpp__pyplusplus_wrapper
#define RenderedStringComponent_hpp__pyplusplus_wrapper

void register_RenderedStringComponent_class();

#endif//RenderedStringComponent_hpp__pyplusplus_wrapper
