// This file has been generated by Py++.

#ifndef TypedPropertyVerticalFormatting_hpp__pyplusplus_wrapper
#define TypedPropertyVerticalFormatting_hpp__pyplusplus_wrapper

void register_TypedPropertyVerticalFormatting_class();

#endif//TypedPropertyVerticalFormatting_hpp__pyplusplus_wrapper
