// This file has been generated by Py++.

#ifndef PropertyDefinitionVector2f_hpp__pyplusplus_wrapper
#define PropertyDefinitionVector2f_hpp__pyplusplus_wrapper

void register_PropertyDefinitionVector2f_class();

#endif//PropertyDefinitionVector2f_hpp__pyplusplus_wrapper
