// This file has been generated by Py++.

#ifndef PropertySet_hpp__pyplusplus_wrapper
#define PropertySet_hpp__pyplusplus_wrapper

void register_PropertySet_class();

#endif//PropertySet_hpp__pyplusplus_wrapper
