// This file has been generated by Py++.

#ifndef NamedXMLResourceManagerFont_hpp__pyplusplus_wrapper
#define NamedXMLResourceManagerFont_hpp__pyplusplus_wrapper

void register_NamedXMLResourceManagerFont_class();

#endif//NamedXMLResourceManagerFont_hpp__pyplusplus_wrapper
