// This file has been generated by Py++.

#ifndef BasicRenderedStringParser_hpp__pyplusplus_wrapper
#define BasicRenderedStringParser_hpp__pyplusplus_wrapper

void register_BasicRenderedStringParser_class();

#endif//BasicRenderedStringParser_hpp__pyplusplus_wrapper
