// This file has been generated by Py++.

#ifndef Clipboard_hpp__pyplusplus_wrapper
#define Clipboard_hpp__pyplusplus_wrapper

void register_Clipboard_class();

#endif//Clipboard_hpp__pyplusplus_wrapper
