// This file has been generated by Py++.

#ifndef AbsoluteDim_hpp__pyplusplus_wrapper
#define AbsoluteDim_hpp__pyplusplus_wrapper

void register_AbsoluteDim_class();

#endif//AbsoluteDim_hpp__pyplusplus_wrapper
