// This file has been generated by Py++.

#ifndef EventAction_hpp__pyplusplus_wrapper
#define EventAction_hpp__pyplusplus_wrapper

void register_EventAction_class();

#endif//EventAction_hpp__pyplusplus_wrapper
