// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColourUBox_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColourUBox_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColourUBox_class();

#endif//FalagardPropertyBaseColourUBox_hpp__pyplusplus_wrapper
