// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColourURect_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColourURect_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColourURect_class();

#endif//PropertyLinkDefinitionColourURect_hpp__pyplusplus_wrapper
