// This file has been generated by Py++.

#ifndef SectionSpecificationVector_hpp__pyplusplus_wrapper
#define SectionSpecificationVector_hpp__pyplusplus_wrapper

void register_SectionSpecificationVector_class();

#endif//SectionSpecificationVector_hpp__pyplusplus_wrapper
