// This file has been generated by Py++.

#ifndef ConstBaseIterator_99d0247943ea008baba14993d63dd301_hpp__pyplusplus_wrapper
#define ConstBaseIterator_99d0247943ea008baba14993d63dd301_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_99d0247943ea008baba14993d63dd301_class();

#endif//ConstBaseIterator_99d0247943ea008baba14993d63dd301_hpp__pyplusplus_wrapper
