// This file has been generated by Py++.

#ifndef RenderingContext_hpp__pyplusplus_wrapper
#define RenderingContext_hpp__pyplusplus_wrapper

void register_RenderingContext_class();

#endif//RenderingContext_hpp__pyplusplus_wrapper
