// This file has been generated by Py++.

#ifndef LinkTargetIterator_hpp__pyplusplus_wrapper
#define LinkTargetIterator_hpp__pyplusplus_wrapper

void register_LinkTargetIterator_class();

#endif//LinkTargetIterator_hpp__pyplusplus_wrapper
