// This file has been generated by Py++.

#ifndef TypedPropertySizef_hpp__pyplusplus_wrapper
#define TypedPropertySizef_hpp__pyplusplus_wrapper

void register_TypedPropertySizef_class();

#endif//TypedPropertySizef_hpp__pyplusplus_wrapper
