// This file has been generated by Py++.

#ifndef ResourceProvider_hpp__pyplusplus_wrapper
#define ResourceProvider_hpp__pyplusplus_wrapper

void register_ResourceProvider_class();

#endif//ResourceProvider_hpp__pyplusplus_wrapper
