// This file has been generated by Py++.

#ifndef OgreImageCodec_hpp__pyplusplus_wrapper
#define OgreImageCodec_hpp__pyplusplus_wrapper

void register_OgreImageCodec_class();

#endif//OgreImageCodec_hpp__pyplusplus_wrapper
