// This file has been generated by Py++.

#ifndef GlobalEventSet_hpp__pyplusplus_wrapper
#define GlobalEventSet_hpp__pyplusplus_wrapper

void register_GlobalEventSet_class();

#endif//GlobalEventSet_hpp__pyplusplus_wrapper
