// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColour_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColour_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColour_class();

#endif//FalagardPropertyBaseColour_hpp__pyplusplus_wrapper
