// This file has been generated by Py++.

#ifndef CentredRenderedString_hpp__pyplusplus_wrapper
#define CentredRenderedString_hpp__pyplusplus_wrapper

void register_CentredRenderedString_class();

#endif//CentredRenderedString_hpp__pyplusplus_wrapper
