// This file has been generated by Py++.

#ifndef PropertyDefinitionBaseMap_hpp__pyplusplus_wrapper
#define PropertyDefinitionBaseMap_hpp__pyplusplus_wrapper

void register_PropertyDefinitionBaseMap_class();

#endif//PropertyDefinitionBaseMap_hpp__pyplusplus_wrapper
