// This file has been generated by Py++.

#ifndef ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472_hpp__pyplusplus_wrapper
#define ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472_class();

#endif//ConstBaseIterator_47ddd968107e89ff0051bf3212fbb472_hpp__pyplusplus_wrapper
