// This file has been generated by Py++.

#ifndef PropertyInitialiser_hpp__pyplusplus_wrapper
#define PropertyInitialiser_hpp__pyplusplus_wrapper

void register_PropertyInitialiser_class();

#endif//PropertyInitialiser_hpp__pyplusplus_wrapper
