// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionFloat_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionFloat_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionFloat_class();

#endif//PropertyLinkDefinitionFloat_hpp__pyplusplus_wrapper
