// This file has been generated by Py++.

#ifndef ResourceEventSet_hpp__pyplusplus_wrapper
#define ResourceEventSet_hpp__pyplusplus_wrapper

void register_ResourceEventSet_class();

#endif//ResourceEventSet_hpp__pyplusplus_wrapper
