// This file has been generated by Py++.

#ifndef PropertyDefinitionBool_hpp__pyplusplus_wrapper
#define PropertyDefinitionBool_hpp__pyplusplus_wrapper

void register_PropertyDefinitionBool_class();

#endif//PropertyDefinitionBool_hpp__pyplusplus_wrapper
