// This file has been generated by Py++.

#ifndef TabControl_hpp__pyplusplus_wrapper
#define TabControl_hpp__pyplusplus_wrapper

void register_TabControl_class();

#endif//TabControl_hpp__pyplusplus_wrapper
