// This file has been generated by Py++.

#ifndef StateIterator_hpp__pyplusplus_wrapper
#define StateIterator_hpp__pyplusplus_wrapper

void register_StateIterator_class();

#endif//StateIterator_hpp__pyplusplus_wrapper
