// This file has been generated by Py++.

#ifndef TypedPropertyVerticalTextFormatting_hpp__pyplusplus_wrapper
#define TypedPropertyVerticalTextFormatting_hpp__pyplusplus_wrapper

void register_TypedPropertyVerticalTextFormatting_class();

#endif//TypedPropertyVerticalTextFormatting_hpp__pyplusplus_wrapper
