// This file has been generated by Py++.

#ifndef PropertyDefinitionSizef_hpp__pyplusplus_wrapper
#define PropertyDefinitionSizef_hpp__pyplusplus_wrapper

void register_PropertyDefinitionSizef_class();

#endif//PropertyDefinitionSizef_hpp__pyplusplus_wrapper
