// This file has been generated by Py++.

#ifndef WindowRendererFactory_hpp__pyplusplus_wrapper
#define WindowRendererFactory_hpp__pyplusplus_wrapper

void register_WindowRendererFactory_class();

#endif//WindowRendererFactory_hpp__pyplusplus_wrapper
