// This file has been generated by Py++.

#ifndef ImagerySection_hpp__pyplusplus_wrapper
#define ImagerySection_hpp__pyplusplus_wrapper

void register_ImagerySection_class();

#endif//ImagerySection_hpp__pyplusplus_wrapper
