// This file has been generated by Py++.

#ifndef SingletonFontManager_hpp__pyplusplus_wrapper
#define SingletonFontManager_hpp__pyplusplus_wrapper

void register_SingletonFontManager_class();

#endif//SingletonFontManager_hpp__pyplusplus_wrapper
