// This file has been generated by Py++.

#ifndef FalagardPropertyBaseUint_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseUint_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseUint_class();

#endif//FalagardPropertyBaseUint_hpp__pyplusplus_wrapper
