// This file has been generated by Py++.

#ifndef PropertyDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper
#define PropertyDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper

void register_PropertyDefinitionVerticalTextFormatting_class();

#endif//PropertyDefinitionVerticalTextFormatting_hpp__pyplusplus_wrapper
