// This file has been generated by Py++.

#ifndef ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6_hpp__pyplusplus_wrapper
#define ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6_class();

#endif//ConstBaseIterator_92b3f0ef60bba0b7bc1aa08b6f3304d6_hpp__pyplusplus_wrapper
