// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColourRect_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColourRect_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColourRect_class();

#endif//FalagardPropertyBaseColourRect_hpp__pyplusplus_wrapper
