// This file has been generated by Py++.

#ifndef GeometryBuffer_hpp__pyplusplus_wrapper
#define GeometryBuffer_hpp__pyplusplus_wrapper

void register_GeometryBuffer_class();

#endif//GeometryBuffer_hpp__pyplusplus_wrapper
