// This file has been generated by Py++.

#ifndef Subscriber_hpp__pyplusplus_wrapper
#define Subscriber_hpp__pyplusplus_wrapper

void register_Subscriber_class();

#endif//Subscriber_hpp__pyplusplus_wrapper
