// This file has been generated by Py++.

#ifndef XMLAttributes_hpp__pyplusplus_wrapper
#define XMLAttributes_hpp__pyplusplus_wrapper

void register_XMLAttributes_class();

#endif//XMLAttributes_hpp__pyplusplus_wrapper
