// This file has been generated by Py++.

#ifndef EventArgs_hpp__pyplusplus_wrapper
#define EventArgs_hpp__pyplusplus_wrapper

void register_EventArgs_class();

#endif//EventArgs_hpp__pyplusplus_wrapper
