// This file has been generated by Py++.

#ifndef PropertyDefinitionRectf_hpp__pyplusplus_wrapper
#define PropertyDefinitionRectf_hpp__pyplusplus_wrapper

void register_PropertyDefinitionRectf_class();

#endif//PropertyDefinitionRectf_hpp__pyplusplus_wrapper
