// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater_.pypp.hpp"

namespace bp = boost::python;

void register_vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__class(){

    { //::std::vector< CEGUI::PropertyDefinitionBase* >
        typedef bp::class_< std::vector< CEGUI::PropertyDefinitionBase* > > vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer_t;
        vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer_t vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer = vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer_t( "vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater_" );
        bp::scope vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__scope( vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer );
        vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__exposer.def( bp::indexing::vector_suite< std::vector< CEGUI::PropertyDefinitionBase* > >::with_policies(bp::return_internal_reference< >()) );
    }

}
