// This file has been generated by Py++.

#ifndef ScriptModule_hpp__pyplusplus_wrapper
#define ScriptModule_hpp__pyplusplus_wrapper

void register_ScriptModule_class();

#endif//ScriptModule_hpp__pyplusplus_wrapper
