// This file has been generated by Py++.

#ifndef ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533_hpp__pyplusplus_wrapper
#define ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533_class();

#endif//ConstBaseIterator_f0585fb3775a7fa1f905d29be5dc1533_hpp__pyplusplus_wrapper
