// This file has been generated by Py++.

#ifndef RegexMatcher_hpp__pyplusplus_wrapper
#define RegexMatcher_hpp__pyplusplus_wrapper

void register_RegexMatcher_class();

#endif//RegexMatcher_hpp__pyplusplus_wrapper
