// This file has been generated by Py++.

#ifndef RawDataContainer_hpp__pyplusplus_wrapper
#define RawDataContainer_hpp__pyplusplus_wrapper

void register_RawDataContainer_class();

#endif//RawDataContainer_hpp__pyplusplus_wrapper
