// This file has been generated by Py++.

#ifndef DragContainer_hpp__pyplusplus_wrapper
#define DragContainer_hpp__pyplusplus_wrapper

void register_DragContainer_class();

#endif//DragContainer_hpp__pyplusplus_wrapper
