// This file has been generated by Py++.

#ifndef Dimension_hpp__pyplusplus_wrapper
#define Dimension_hpp__pyplusplus_wrapper

void register_Dimension_class();

#endif//Dimension_hpp__pyplusplus_wrapper
