// This file has been generated by Py++.

#ifndef PropertyIterator_hpp__pyplusplus_wrapper
#define PropertyIterator_hpp__pyplusplus_wrapper

void register_PropertyIterator_class();

#endif//PropertyIterator_hpp__pyplusplus_wrapper
