// This file has been generated by Py++.

#ifndef PropertyDefinitionFont_hpp__pyplusplus_wrapper
#define PropertyDefinitionFont_hpp__pyplusplus_wrapper

void register_PropertyDefinitionFont_class();

#endif//PropertyDefinitionFont_hpp__pyplusplus_wrapper
