// This file has been generated by Py++.

#ifndef TypedPropertyColourUVector2_hpp__pyplusplus_wrapper
#define TypedPropertyColourUVector2_hpp__pyplusplus_wrapper

void register_TypedPropertyColourUVector2_class();

#endif//TypedPropertyColourUVector2_hpp__pyplusplus_wrapper
