// This file has been generated by Py++.

#ifndef FalagardPropertyBaseString_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseString_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseString_class();

#endif//FalagardPropertyBaseString_hpp__pyplusplus_wrapper
