// This file has been generated by Py++.

#ifndef ItemListBase_hpp__pyplusplus_wrapper
#define ItemListBase_hpp__pyplusplus_wrapper

void register_ItemListBase_class();

#endif//ItemListBase_hpp__pyplusplus_wrapper
