// This file has been generated by Py++.

#ifndef TextComponent_hpp__pyplusplus_wrapper
#define TextComponent_hpp__pyplusplus_wrapper

void register_TextComponent_class();

#endif//TextComponent_hpp__pyplusplus_wrapper
