// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColour_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColour_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColour_class();

#endif//PropertyLinkDefinitionColour_hpp__pyplusplus_wrapper
