// This file has been generated by Py++.

#ifndef LinkedEventArgs_hpp__pyplusplus_wrapper
#define LinkedEventArgs_hpp__pyplusplus_wrapper

void register_LinkedEventArgs_class();

#endif//LinkedEventArgs_hpp__pyplusplus_wrapper
