// This file has been generated by Py++.

#ifndef CoordConverter_hpp__pyplusplus_wrapper
#define CoordConverter_hpp__pyplusplus_wrapper

void register_CoordConverter_class();

#endif//CoordConverter_hpp__pyplusplus_wrapper
