// This file has been generated by Py++.

#ifndef ComponentList_hpp__pyplusplus_wrapper
#define ComponentList_hpp__pyplusplus_wrapper

void register_ComponentList_class();

#endif//ComponentList_hpp__pyplusplus_wrapper
