// This file has been generated by Py++.

#ifndef Quaternion_hpp__pyplusplus_wrapper
#define Quaternion_hpp__pyplusplus_wrapper

void register_Quaternion_class();

#endif//Quaternion_hpp__pyplusplus_wrapper
