// This file has been generated by Py++.

#ifndef RenderingSurface_hpp__pyplusplus_wrapper
#define RenderingSurface_hpp__pyplusplus_wrapper

void register_RenderingSurface_class();

#endif//RenderingSurface_hpp__pyplusplus_wrapper
