// This file has been generated by Py++.

#ifndef XMLParser_hpp__pyplusplus_wrapper
#define XMLParser_hpp__pyplusplus_wrapper

void register_XMLParser_class();

#endif//XMLParser_hpp__pyplusplus_wrapper
