// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionHorizontalTextFormatting_class();

#endif//PropertyLinkDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper
