// This file has been generated by Py++.

#ifndef ResourceEventArgs_hpp__pyplusplus_wrapper
#define ResourceEventArgs_hpp__pyplusplus_wrapper

void register_ResourceEventArgs_class();

#endif//ResourceEventArgs_hpp__pyplusplus_wrapper
