// This file has been generated by Py++.

#ifndef FontManager_hpp__pyplusplus_wrapper
#define FontManager_hpp__pyplusplus_wrapper

void register_FontManager_class();

#endif//FontManager_hpp__pyplusplus_wrapper
