// This file has been generated by Py++.

#ifndef KeyEventArgs_hpp__pyplusplus_wrapper
#define KeyEventArgs_hpp__pyplusplus_wrapper

void register_KeyEventArgs_class();

#endif//KeyEventArgs_hpp__pyplusplus_wrapper
