// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColourRect_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColourRect_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColourRect_class();

#endif//PropertyLinkDefinitionColourRect_hpp__pyplusplus_wrapper
