// This file has been generated by Py++.

#ifndef FalagardPropertyBaseVector2f_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseVector2f_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseVector2f_class();

#endif//FalagardPropertyBaseVector2f_hpp__pyplusplus_wrapper
