// This file has been generated by Py++.

#ifndef PropertyDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper
#define PropertyDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper

void register_PropertyDefinitionHorizontalTextFormatting_class();

#endif//PropertyDefinitionHorizontalTextFormatting_hpp__pyplusplus_wrapper
