// This file has been generated by Py++.

#ifndef TypedPropertyFont_hpp__pyplusplus_wrapper
#define TypedPropertyFont_hpp__pyplusplus_wrapper

void register_TypedPropertyFont_class();

#endif//TypedPropertyFont_hpp__pyplusplus_wrapper
