// This file has been generated by Py++.

#ifndef FalagardPropertyBaseVerticalTextFormatting_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseVerticalTextFormatting_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseVerticalTextFormatting_class();

#endif//FalagardPropertyBaseVerticalTextFormatting_hpp__pyplusplus_wrapper
