// This file has been generated by Py++.

#ifndef WidgetComponentIterator_hpp__pyplusplus_wrapper
#define WidgetComponentIterator_hpp__pyplusplus_wrapper

void register_WidgetComponentIterator_class();

#endif//WidgetComponentIterator_hpp__pyplusplus_wrapper
