// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/map.hpp"
#include "generators/include/python_CEGUI.h"
#include "ImagerySectionMap.pypp.hpp"

namespace bp = boost::python;

void register_ImagerySectionMap_class(){

    { //::std::map<CEGUI::String, CEGUI::ImagerySection*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::ImagerySection*> > >
        typedef bp::class_< std::map<CEGUI::String, CEGUI::ImagerySection*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::ImagerySection*> > > > ImagerySectionMap_exposer_t;
        ImagerySectionMap_exposer_t ImagerySectionMap_exposer = ImagerySectionMap_exposer_t( "ImagerySectionMap" );
        bp::scope ImagerySectionMap_scope( ImagerySectionMap_exposer );
        ImagerySectionMap_exposer.def( bp::indexing::map_suite< std::map<CEGUI::String, CEGUI::ImagerySection*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::ImagerySection*> > > >() );
    }

}
