// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColourUBox_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColourUBox_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColourUBox_class();

#endif//PropertyLinkDefinitionColourUBox_hpp__pyplusplus_wrapper
