// This file has been generated by Py++.

#ifndef TypedPropertyColourURect_hpp__pyplusplus_wrapper
#define TypedPropertyColourURect_hpp__pyplusplus_wrapper

void register_TypedPropertyColourURect_class();

#endif//TypedPropertyColourURect_hpp__pyplusplus_wrapper
