// This file has been generated by Py++.

#ifndef FrameComponentIterator_hpp__pyplusplus_wrapper
#define FrameComponentIterator_hpp__pyplusplus_wrapper

void register_FrameComponentIterator_class();

#endif//FrameComponentIterator_hpp__pyplusplus_wrapper
