// This file has been generated by Py++.

#ifndef TreeEventArgs_hpp__pyplusplus_wrapper
#define TreeEventArgs_hpp__pyplusplus_wrapper

void register_TreeEventArgs_class();

#endif//TreeEventArgs_hpp__pyplusplus_wrapper
