// This file has been generated by Py++.

#ifndef ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb_hpp__pyplusplus_wrapper
#define ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb_class();

#endif//ConstBaseIterator_6318b664b630c5a4e45127a69c6748cb_hpp__pyplusplus_wrapper
