// This file has been generated by Py++.

#ifndef XMLSerializer_hpp__pyplusplus_wrapper
#define XMLSerializer_hpp__pyplusplus_wrapper

void register_XMLSerializer_class();

#endif//XMLSerializer_hpp__pyplusplus_wrapper
