// This file has been generated by Py++.

#ifndef SingletonLogger_hpp__pyplusplus_wrapper
#define SingletonLogger_hpp__pyplusplus_wrapper

void register_SingletonLogger_class();

#endif//SingletonLogger_hpp__pyplusplus_wrapper
