// This file has been generated by Py++.

#ifndef RenderQueue_hpp__pyplusplus_wrapper
#define RenderQueue_hpp__pyplusplus_wrapper

void register_RenderQueue_class();

#endif//RenderQueue_hpp__pyplusplus_wrapper
