// This file has been generated by Py++.

#ifndef ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b_hpp__pyplusplus_wrapper
#define ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b_class();

#endif//ConstBaseIterator_9dce73c5f5e3a55d1a66a6e740fbf48b_hpp__pyplusplus_wrapper
