// This file has been generated by Py++.

#ifndef ButtonBase_hpp__pyplusplus_wrapper
#define ButtonBase_hpp__pyplusplus_wrapper

void register_ButtonBase_class();

#endif//ButtonBase_hpp__pyplusplus_wrapper
