// This file has been generated by Py++.

#ifndef ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9_hpp__pyplusplus_wrapper
#define ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9_class();

#endif//ConstBaseIterator_78cd23dcc51d261e1f508a899854e1a9_hpp__pyplusplus_wrapper
