// This file has been generated by Py++.

#ifndef ScrolledItemListBase_hpp__pyplusplus_wrapper
#define ScrolledItemListBase_hpp__pyplusplus_wrapper

void register_ScrolledItemListBase_class();

#endif//ScrolledItemListBase_hpp__pyplusplus_wrapper
