// This file has been generated by Py++.

#ifndef ImagePropertyDim_hpp__pyplusplus_wrapper
#define ImagePropertyDim_hpp__pyplusplus_wrapper

void register_ImagePropertyDim_class();

#endif//ImagePropertyDim_hpp__pyplusplus_wrapper
