// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionUint_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionUint_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionUint_class();

#endif//PropertyLinkDefinitionUint_hpp__pyplusplus_wrapper
