// This file has been generated by Py++.

#ifndef ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd_hpp__pyplusplus_wrapper
#define ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd_class();

#endif//ConstBaseIterator_d0a5fcfab7d8e4121b568ec58e88f4fd_hpp__pyplusplus_wrapper
