// This file has been generated by Py++.

#ifndef WidgetLookManager_hpp__pyplusplus_wrapper
#define WidgetLookManager_hpp__pyplusplus_wrapper

void register_WidgetLookManager_class();

#endif//WidgetLookManager_hpp__pyplusplus_wrapper
