// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/map.hpp"
#include "generators/include/python_CEGUI.h"
#include "PropertyDefinitionBaseMap.pypp.hpp"

namespace bp = boost::python;

void register_PropertyDefinitionBaseMap_class(){

    { //::std::map<CEGUI::String, CEGUI::PropertyDefinitionBase*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::PropertyDefinitionBase*> > >
        typedef bp::class_< std::map<CEGUI::String, CEGUI::PropertyDefinitionBase*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::PropertyDefinitionBase*> > > > PropertyDefinitionBaseMap_exposer_t;
        PropertyDefinitionBaseMap_exposer_t PropertyDefinitionBaseMap_exposer = PropertyDefinitionBaseMap_exposer_t( "PropertyDefinitionBaseMap" );
        bp::scope PropertyDefinitionBaseMap_scope( PropertyDefinitionBaseMap_exposer );
        PropertyDefinitionBaseMap_exposer.def( bp::indexing::map_suite< std::map<CEGUI::String, CEGUI::PropertyDefinitionBase*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::PropertyDefinitionBase*> > > >() );
    }

}
