// This file has been generated by Py++.

#ifndef TabButton_hpp__pyplusplus_wrapper
#define TabButton_hpp__pyplusplus_wrapper

void register_TabButton_class();

#endif//TabButton_hpp__pyplusplus_wrapper
