// This file has been generated by Py++.

#ifndef PropertyInitialiserMap_hpp__pyplusplus_wrapper
#define PropertyInitialiserMap_hpp__pyplusplus_wrapper

void register_PropertyInitialiserMap_class();

#endif//PropertyInitialiserMap_hpp__pyplusplus_wrapper
