// This file has been generated by Py++.

#ifndef WidgetDim_hpp__pyplusplus_wrapper
#define WidgetDim_hpp__pyplusplus_wrapper

void register_WidgetDim_class();

#endif//WidgetDim_hpp__pyplusplus_wrapper
