// This file has been generated by Py++.

#ifndef ElementEventArgs_hpp__pyplusplus_wrapper
#define ElementEventArgs_hpp__pyplusplus_wrapper

void register_ElementEventArgs_class();

#endif//ElementEventArgs_hpp__pyplusplus_wrapper
