// This file has been generated by Py++.

#ifndef NamedAreaIterator_hpp__pyplusplus_wrapper
#define NamedAreaIterator_hpp__pyplusplus_wrapper

void register_NamedAreaIterator_class();

#endif//NamedAreaIterator_hpp__pyplusplus_wrapper
