// This file has been generated by Py++.

#ifndef OpenGLRenderer_hpp__pyplusplus_wrapper
#define OpenGLRenderer_hpp__pyplusplus_wrapper

void register_OpenGLRenderer_class();

#endif//OpenGLRenderer_hpp__pyplusplus_wrapper
