// This file has been generated by Py++.

#ifndef StdPairFloatFloat_hpp__pyplusplus_wrapper
#define StdPairFloatFloat_hpp__pyplusplus_wrapper

void register_StdPairFloatFloat_class();

#endif//StdPairFloatFloat_hpp__pyplusplus_wrapper
