// This file has been generated by Py++.

#ifndef RenderingWindow_hpp__pyplusplus_wrapper
#define RenderingWindow_hpp__pyplusplus_wrapper

void register_RenderingWindow_class();

#endif//RenderingWindow_hpp__pyplusplus_wrapper
