// This file has been generated by Py++.

#ifndef SingletonWindowRendererManager_hpp__pyplusplus_wrapper
#define SingletonWindowRendererManager_hpp__pyplusplus_wrapper

void register_SingletonWindowRendererManager_class();

#endif//SingletonWindowRendererManager_hpp__pyplusplus_wrapper
