// This file has been generated by Py++.

#ifndef MouseCursorEventArgs_hpp__pyplusplus_wrapper
#define MouseCursorEventArgs_hpp__pyplusplus_wrapper

void register_MouseCursorEventArgs_class();

#endif//MouseCursorEventArgs_hpp__pyplusplus_wrapper
