// This file has been generated by Py++.

#ifndef ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d_hpp__pyplusplus_wrapper
#define ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d_class();

#endif//ConstBaseIterator_6cea82a17f94b95b10c333412f843a6d_hpp__pyplusplus_wrapper
