// This file has been generated by Py++.

#ifndef BasicImage_hpp__pyplusplus_wrapper
#define BasicImage_hpp__pyplusplus_wrapper

void register_BasicImage_class();

#endif//BasicImage_hpp__pyplusplus_wrapper
