// This file has been generated by Py++.

#ifndef AnimationInstance_hpp__pyplusplus_wrapper
#define AnimationInstance_hpp__pyplusplus_wrapper

void register_AnimationInstance_class();

#endif//AnimationInstance_hpp__pyplusplus_wrapper
