// This file has been generated by Py++.

#ifndef LayerIterator_hpp__pyplusplus_wrapper
#define LayerIterator_hpp__pyplusplus_wrapper

void register_LayerIterator_class();

#endif//LayerIterator_hpp__pyplusplus_wrapper
