// This file has been generated by Py++.

#ifndef StringSet_hpp__pyplusplus_wrapper
#define StringSet_hpp__pyplusplus_wrapper

void register_StringSet_class();

#endif//StringSet_hpp__pyplusplus_wrapper
