// This file has been generated by Py++.

#ifndef PropertyDefinitionColourUVector2_hpp__pyplusplus_wrapper
#define PropertyDefinitionColourUVector2_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColourUVector2_class();

#endif//PropertyDefinitionColourUVector2_hpp__pyplusplus_wrapper
