// This file has been generated by Py++.

#ifndef ScrollablePane_hpp__pyplusplus_wrapper
#define ScrollablePane_hpp__pyplusplus_wrapper

void register_ScrollablePane_class();

#endif//ScrollablePane_hpp__pyplusplus_wrapper
