// This file has been generated by Py++.

#ifndef OpenGL3Renderer_hpp__pyplusplus_wrapper
#define OpenGL3Renderer_hpp__pyplusplus_wrapper

void register_OpenGL3Renderer_class();

#endif//OpenGL3Renderer_hpp__pyplusplus_wrapper
