// This file has been generated by Py++.

#ifndef PropertyDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper
#define PropertyDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper

void register_PropertyDefinitionHorizontalFormatting_class();

#endif//PropertyDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper
