// This file has been generated by Py++.

#ifndef ListHeaderSegment_hpp__pyplusplus_wrapper
#define ListHeaderSegment_hpp__pyplusplus_wrapper

void register_ListHeaderSegment_class();

#endif//ListHeaderSegment_hpp__pyplusplus_wrapper
