// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "Vector2f.pypp.hpp"

namespace bp = boost::python;

void register_Vector2f_class(){

    { //::CEGUI::Vector2< float >
        typedef bp::class_< CEGUI::Vector2< float > > Vector2f_exposer_t;
        Vector2f_exposer_t Vector2f_exposer = Vector2f_exposer_t( "Vector2f", bp::init< >() );
        bp::scope Vector2f_scope( Vector2f_exposer );
        Vector2f_exposer.def( bp::init< float, float >(( bp::arg("x"), bp::arg("y") )) );
        Vector2f_exposer.def( bp::init< CEGUI::Vector2< float > const & >(( bp::arg("v") )) );
        { //::CEGUI::Vector2< float >::one
        
            typedef CEGUI::Vector2< float > exported_class_t;
            typedef ::CEGUI::Vector2< float > ( *one_function_type )(  );
            
            Vector2f_exposer.def( 
                "one"
                , one_function_type( &::CEGUI::Vector2< float >::one )
                , "!  finger saving alias for Vector2(1, 1)\n" );
        
        }
        { //::CEGUI::Vector2< float >::one_x
        
            typedef CEGUI::Vector2< float > exported_class_t;
            typedef ::CEGUI::Vector2< float > ( *one_x_function_type )(  );
            
            Vector2f_exposer.def( 
                "one_x"
                , one_x_function_type( &::CEGUI::Vector2< float >::one_x )
                , "!  finger saving alias for Vector2(1, 0)\n" );
        
        }
        { //::CEGUI::Vector2< float >::one_y
        
            typedef CEGUI::Vector2< float > exported_class_t;
            typedef ::CEGUI::Vector2< float > ( *one_y_function_type )(  );
            
            Vector2f_exposer.def( 
                "one_y"
                , one_y_function_type( &::CEGUI::Vector2< float >::one_y )
                , "!  finger saving alias for Vector2(0, 1)\n" );
        
        }
        Vector2f_exposer.def( bp::self != bp::self );
        Vector2f_exposer.def( bp::self * bp::self );
        Vector2f_exposer.def( bp::self * bp::other< float >() );
        Vector2f_exposer.def( bp::self *= bp::self );
        Vector2f_exposer.def( bp::self *= bp::other< float >() );
        Vector2f_exposer.def( bp::self + bp::self );
        Vector2f_exposer.def( bp::self += bp::self );
        Vector2f_exposer.def( bp::self - bp::self );
        Vector2f_exposer.def( bp::self -= bp::self );
        Vector2f_exposer.def( bp::self / bp::self );
        Vector2f_exposer.def( bp::self / bp::other< float >() );
        Vector2f_exposer.def( bp::self /= bp::self );
        Vector2f_exposer.def( bp::self == bp::self );
        { //::CEGUI::Vector2< float >::zero
        
            typedef CEGUI::Vector2< float > exported_class_t;
            typedef ::CEGUI::Vector2< float > ( *zero_function_type )(  );
            
            Vector2f_exposer.def( 
                "zero"
                , zero_function_type( &::CEGUI::Vector2< float >::zero )
                , "!  finger saving alias for Vector2(0, 0)\n" );
        
        }
        Vector2f_exposer.def_readwrite( "d_x", &CEGUI::Vector2< float >::d_x );
        Vector2f_exposer.def_readwrite( "d_y", &CEGUI::Vector2< float >::d_y );
        Vector2f_exposer.staticmethod( "one" );
        Vector2f_exposer.staticmethod( "one_x" );
        Vector2f_exposer.staticmethod( "one_y" );
        Vector2f_exposer.staticmethod( "zero" );
    }

}
