// This file has been generated by Py++.

#ifndef ImageManager_hpp__pyplusplus_wrapper
#define ImageManager_hpp__pyplusplus_wrapper

void register_ImageManager_class();

#endif//ImageManager_hpp__pyplusplus_wrapper
