// This file has been generated by Py++.

#ifndef ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec_hpp__pyplusplus_wrapper
#define ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec_class();

#endif//ConstBaseIterator_6e18222775ccb39f7ec1f5c112d444ec_hpp__pyplusplus_wrapper
