// This file has been generated by Py++.

#ifndef vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__hpp__pyplusplus_wrapper
#define vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__hpp__pyplusplus_wrapper

void register_vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__class();

#endif//vector_less__CEGUI_scope_PropertyDefinitionBase_ptr___greater__hpp__pyplusplus_wrapper
