// This file has been generated by Py++.

#ifndef LayerSpecification_hpp__pyplusplus_wrapper
#define LayerSpecification_hpp__pyplusplus_wrapper

void register_LayerSpecification_class();

#endif//LayerSpecification_hpp__pyplusplus_wrapper
