// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "ConstBaseIterator_99d0247943ea008baba14993d63dd301.pypp.hpp"

namespace bp = boost::python;

struct ConstBaseIterator_less__std_scope_vector_less__CEGUI_scope_SectionSpecification__greater__comma__CEGUI_scope_SectionSpecification__greater__wrapper : CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >, bp::wrapper< CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > > {

    ConstBaseIterator_less__std_scope_vector_less__CEGUI_scope_SectionSpecification__greater__comma__CEGUI_scope_SectionSpecification__greater__wrapper( )
    : CEGUI::ConstBaseIterator<std::vector<CEGUI::SectionSpecification, std::allocator<CEGUI::SectionSpecification> >, CEGUI::SectionSpecification>( )
      , bp::wrapper< CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > >(){
        // null constructor
    
    }

    virtual ::CEGUI::SectionSpecification getCurrentValue(  ) const {
        bp::override func_getCurrentValue = this->get_override( "getCurrentValue" );
        return func_getCurrentValue(  );
    }

};

void register_ConstBaseIterator_99d0247943ea008baba14993d63dd301_class(){

    { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >
        typedef bp::class_< ConstBaseIterator_less__std_scope_vector_less__CEGUI_scope_SectionSpecification__greater__comma__CEGUI_scope_SectionSpecification__greater__wrapper, boost::noncopyable > ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer_t;
        ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer_t ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer = ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer_t( "ConstBaseIterator_99d0247943ea008baba14993d63dd301", bp::no_init );
        bp::scope ConstBaseIterator_99d0247943ea008baba14993d63dd301_scope( ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer );
        ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( bp::init< >("*************************************************************************\n\
           No default construction available\n\
        *************************************************************************\n") );
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::getCurrentValue
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef ::CEGUI::SectionSpecification ( exported_class_t::*getCurrentValue_function_type )(  ) const;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "getCurrentValue"
                , bp::pure_virtual( getCurrentValue_function_type(&::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::getCurrentValue) )
                , "*!\n\
            \n\
               Return the value for the item at the current iterator position.\n\
            *\n" );
        
        }
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::isAtEnd
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef bool ( exported_class_t::*isAtEnd_function_type )(  ) const;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "isAtEnd"
                , isAtEnd_function_type( &::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::isAtEnd )
                , "*!\n\
            \n\
               Return whether the current iterator position is at the end of the iterators range.\n\
            *\n" );
        
        }
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::isAtStart
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef bool ( exported_class_t::*isAtStart_function_type )(  ) const;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "isAtStart"
                , isAtStart_function_type( &::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::isAtStart )
                , "*!\n\
            \n\
               Return whether the current iterator position is at the start of the iterators range.\n\
            *\n" );
        
        }
        ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( bp::self != bp::self );
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::operator=
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef ::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > & ( exported_class_t::*assign_function_type )( ::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > const & ) ;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "assign"
                , assign_function_type( &::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::operator= )
                , ( bp::arg("rhs") )
                , bp::return_self< >()
                , "*!\n\
            \n\
               ConstBaseIterator assignment operator\n\
            *\n" );
        
        }
        ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( bp::self == bp::self );
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::toEnd
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef void ( exported_class_t::*toEnd_function_type )(  ) ;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "toEnd"
                , toEnd_function_type( &::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::toEnd )
                , "*!\n\
            \n\
               Set the iterator current position to the end position.\n\
            *\n" );
        
        }
        { //::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::toStart
        
            typedef CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification > exported_class_t;
            typedef void ( exported_class_t::*toStart_function_type )(  ) ;
            
            ConstBaseIterator_99d0247943ea008baba14993d63dd301_exposer.def( 
                "toStart"
                , toStart_function_type( &::CEGUI::ConstBaseIterator< std::vector< CEGUI::SectionSpecification >, CEGUI::SectionSpecification >::toStart )
                , "*!\n\
            \n\
               Set the iterator current position to the start position.\n\
            *\n" );
        
        }
    }

}
