// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionString_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionString_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionString_class();

#endif//PropertyLinkDefinitionString_hpp__pyplusplus_wrapper
