// This file has been generated by Py++.

#ifndef SectionIterator_hpp__pyplusplus_wrapper
#define SectionIterator_hpp__pyplusplus_wrapper

void register_SectionIterator_class();

#endif//SectionIterator_hpp__pyplusplus_wrapper
