// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColourUDim_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColourUDim_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColourUDim_class();

#endif//PropertyLinkDefinitionColourUDim_hpp__pyplusplus_wrapper
