// This file has been generated by Py++.

#ifndef PopupMenu_hpp__pyplusplus_wrapper
#define PopupMenu_hpp__pyplusplus_wrapper

void register_PopupMenu_class();

#endif//PopupMenu_hpp__pyplusplus_wrapper
