// This file has been generated by Py++.

#ifndef DragDropEventArgs_hpp__pyplusplus_wrapper
#define DragDropEventArgs_hpp__pyplusplus_wrapper

void register_DragDropEventArgs_class();

#endif//DragDropEventArgs_hpp__pyplusplus_wrapper
