// This file has been generated by Py++.

#ifndef FalagardPropertyBaseHorizontalFormatting_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseHorizontalFormatting_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseHorizontalFormatting_class();

#endif//FalagardPropertyBaseHorizontalFormatting_hpp__pyplusplus_wrapper
