// This file has been generated by Py++.

#ifndef MCLGridRef_hpp__pyplusplus_wrapper
#define MCLGridRef_hpp__pyplusplus_wrapper

void register_MCLGridRef_class();

#endif//MCLGridRef_hpp__pyplusplus_wrapper
