// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionBool_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionBool_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionBool_class();

#endif//PropertyLinkDefinitionBool_hpp__pyplusplus_wrapper
