// This file has been generated by Py++.

#ifndef TypedPropertyHorizontalFormatting_hpp__pyplusplus_wrapper
#define TypedPropertyHorizontalFormatting_hpp__pyplusplus_wrapper

void register_TypedPropertyHorizontalFormatting_class();

#endif//TypedPropertyHorizontalFormatting_hpp__pyplusplus_wrapper
