// This file has been generated by Py++.

#ifndef ConstBaseIterator_937e880515ad590c17d4b2b82a347310_hpp__pyplusplus_wrapper
#define ConstBaseIterator_937e880515ad590c17d4b2b82a347310_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_937e880515ad590c17d4b2b82a347310_class();

#endif//ConstBaseIterator_937e880515ad590c17d4b2b82a347310_hpp__pyplusplus_wrapper
