// This file has been generated by Py++.

#ifndef PropertyDefinitionBase_hpp__pyplusplus_wrapper
#define PropertyDefinitionBase_hpp__pyplusplus_wrapper

void register_PropertyDefinitionBase_class();

#endif//PropertyDefinitionBase_hpp__pyplusplus_wrapper
