// This file has been generated by Py++.

#ifndef ConstBaseIterator_7f0012c4899354432f095ef9bb577046_hpp__pyplusplus_wrapper
#define ConstBaseIterator_7f0012c4899354432f095ef9bb577046_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_7f0012c4899354432f095ef9bb577046_class();

#endif//ConstBaseIterator_7f0012c4899354432f095ef9bb577046_hpp__pyplusplus_wrapper
