// This file has been generated by Py++.

#ifndef Workarounds_hpp__pyplusplus_wrapper
#define Workarounds_hpp__pyplusplus_wrapper

void register_Workarounds_class();

#endif//Workarounds_hpp__pyplusplus_wrapper
