// This file has been generated by Py++.

#ifndef SequentialLayoutContainer_hpp__pyplusplus_wrapper
#define SequentialLayoutContainer_hpp__pyplusplus_wrapper

void register_SequentialLayoutContainer_class();

#endif//SequentialLayoutContainer_hpp__pyplusplus_wrapper
