// This file has been generated by Py++.

#ifndef RenderedStringImageComponent_hpp__pyplusplus_wrapper
#define RenderedStringImageComponent_hpp__pyplusplus_wrapper

void register_RenderedStringImageComponent_class();

#endif//RenderedStringImageComponent_hpp__pyplusplus_wrapper
