// This file has been generated by Py++.

#ifndef HorizontalLayoutContainer_hpp__pyplusplus_wrapper
#define HorizontalLayoutContainer_hpp__pyplusplus_wrapper

void register_HorizontalLayoutContainer_class();

#endif//HorizontalLayoutContainer_hpp__pyplusplus_wrapper
