// This file has been generated by Py++.

#ifndef PropertyInitialiserVector_hpp__pyplusplus_wrapper
#define PropertyInitialiserVector_hpp__pyplusplus_wrapper

void register_PropertyInitialiserVector_class();

#endif//PropertyInitialiserVector_hpp__pyplusplus_wrapper
