// This file has been generated by Py++.

#ifndef EventLinkDefinitionIterator_hpp__pyplusplus_wrapper
#define EventLinkDefinitionIterator_hpp__pyplusplus_wrapper

void register_EventLinkDefinitionIterator_class();

#endif//EventLinkDefinitionIterator_hpp__pyplusplus_wrapper
