// This file has been generated by Py++.

#ifndef ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f_hpp__pyplusplus_wrapper
#define ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f_class();

#endif//ConstBaseIterator_2c9936f57a2ad030e0dfadb6e79a326f_hpp__pyplusplus_wrapper
