// This file has been generated by Py++.

#ifndef ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174_hpp__pyplusplus_wrapper
#define ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174_class();

#endif//ConstBaseIterator_6da79d61dc2d4c154ee9113f9d44d174_hpp__pyplusplus_wrapper
