// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColourURect_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColourURect_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColourURect_class();

#endif//FalagardPropertyBaseColourURect_hpp__pyplusplus_wrapper
