// This file has been generated by Py++.

#ifndef SingletonWidgetLookManager_hpp__pyplusplus_wrapper
#define SingletonWidgetLookManager_hpp__pyplusplus_wrapper

void register_SingletonWidgetLookManager_class();

#endif//SingletonWidgetLookManager_hpp__pyplusplus_wrapper
