// This file has been generated by Py++.

#ifndef LoadableUIElementIterator_hpp__pyplusplus_wrapper
#define LoadableUIElementIterator_hpp__pyplusplus_wrapper

void register_LoadableUIElementIterator_class();

#endif//LoadableUIElementIterator_hpp__pyplusplus_wrapper
