// This file has been generated by Py++.

#ifndef ListHeader_hpp__pyplusplus_wrapper
#define ListHeader_hpp__pyplusplus_wrapper

void register_ListHeader_class();

#endif//ListHeader_hpp__pyplusplus_wrapper
