// This file has been generated by Py++.

#ifndef ToggleButton_hpp__pyplusplus_wrapper
#define ToggleButton_hpp__pyplusplus_wrapper

void register_ToggleButton_class();

#endif//ToggleButton_hpp__pyplusplus_wrapper
