// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionHorizontalFormatting_class();

#endif//PropertyLinkDefinitionHorizontalFormatting_hpp__pyplusplus_wrapper
