// This file has been generated by Py++.

#ifndef RenderedStringWidgetComponent_hpp__pyplusplus_wrapper
#define RenderedStringWidgetComponent_hpp__pyplusplus_wrapper

void register_RenderedStringWidgetComponent_class();

#endif//RenderedStringWidgetComponent_hpp__pyplusplus_wrapper
