// This file has been generated by Py++.

#ifndef EventLinkDefinition_hpp__pyplusplus_wrapper
#define EventLinkDefinition_hpp__pyplusplus_wrapper

void register_EventLinkDefinition_class();

#endif//EventLinkDefinition_hpp__pyplusplus_wrapper
