// This file has been generated by Py++.

#ifndef DefaultWindow_hpp__pyplusplus_wrapper
#define DefaultWindow_hpp__pyplusplus_wrapper

void register_DefaultWindow_class();

#endif//DefaultWindow_hpp__pyplusplus_wrapper
