// This file has been generated by Py++.

#ifndef SimpleTimer_hpp__pyplusplus_wrapper
#define SimpleTimer_hpp__pyplusplus_wrapper

void register_SimpleTimer_class();

#endif//SimpleTimer_hpp__pyplusplus_wrapper
