// This file has been generated by Py++.

#ifndef UnifiedDim_hpp__pyplusplus_wrapper
#define UnifiedDim_hpp__pyplusplus_wrapper

void register_UnifiedDim_class();

#endif//UnifiedDim_hpp__pyplusplus_wrapper
