// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "RenderedStringParser.pypp.hpp"

namespace bp = boost::python;

struct RenderedStringParser_wrapper : CEGUI::RenderedStringParser, bp::wrapper< CEGUI::RenderedStringParser > {

    RenderedStringParser_wrapper()
    : CEGUI::RenderedStringParser()
      , bp::wrapper< CEGUI::RenderedStringParser >(){
        // null constructor
        
    }

    virtual ::CEGUI::RenderedString parse( ::CEGUI::String const & input_string, ::CEGUI::Font const * initial_font, ::CEGUI::ColourRect const * initial_colours ){
        bp::override func_parse = this->get_override( "parse" );
        return func_parse( boost::ref(input_string), boost::python::ptr(initial_font), boost::python::ptr(initial_colours) );
    }

};

void register_RenderedStringParser_class(){

    { //::CEGUI::RenderedStringParser
        typedef bp::class_< RenderedStringParser_wrapper, boost::noncopyable > RenderedStringParser_exposer_t;
        RenderedStringParser_exposer_t RenderedStringParser_exposer = RenderedStringParser_exposer_t( "RenderedStringParser" );
        bp::scope RenderedStringParser_scope( RenderedStringParser_exposer );
        { //::CEGUI::RenderedStringParser::parse
        
            typedef ::CEGUI::RenderedString ( ::CEGUI::RenderedStringParser::*parse_function_type )( ::CEGUI::String const &,::CEGUI::Font const *,::CEGUI::ColourRect const * ) ;
            
            RenderedStringParser_exposer.def( 
                "parse"
                , bp::pure_virtual( parse_function_type(&::CEGUI::RenderedStringParser::parse) )
                , ( bp::arg("input_string"), bp::arg("initial_font"), bp::arg("initial_colours") ) );
        
        }
    }

}
