// This file has been generated by Py++.

#ifndef SingletonImageManager_hpp__pyplusplus_wrapper
#define SingletonImageManager_hpp__pyplusplus_wrapper

void register_SingletonImageManager_class();

#endif//SingletonImageManager_hpp__pyplusplus_wrapper
