// This file has been generated by Py++.

#ifndef RenderTargetEventArgs_hpp__pyplusplus_wrapper
#define RenderTargetEventArgs_hpp__pyplusplus_wrapper

void register_RenderTargetEventArgs_class();

#endif//RenderTargetEventArgs_hpp__pyplusplus_wrapper
