// This file has been generated by Py++.

#ifndef TypeAliasIterator_hpp__pyplusplus_wrapper
#define TypeAliasIterator_hpp__pyplusplus_wrapper

void register_TypeAliasIterator_class();

#endif//TypeAliasIterator_hpp__pyplusplus_wrapper
