// This file has been generated by Py++.

#ifndef FalagardPropertyBaseHorizontalTextFormatting_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseHorizontalTextFormatting_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseHorizontalTextFormatting_class();

#endif//FalagardPropertyBaseHorizontalTextFormatting_hpp__pyplusplus_wrapper
