// This file has been generated by Py++.

#ifndef WindowRendererManager_hpp__pyplusplus_wrapper
#define WindowRendererManager_hpp__pyplusplus_wrapper

void register_WindowRendererManager_class();

#endif//WindowRendererManager_hpp__pyplusplus_wrapper
