// This file has been generated by Py++.

#ifndef ListboxTextItem_hpp__pyplusplus_wrapper
#define ListboxTextItem_hpp__pyplusplus_wrapper

void register_ListboxTextItem_class();

#endif//ListboxTextItem_hpp__pyplusplus_wrapper
