// This file has been generated by Py++.

#ifndef RenderedString_hpp__pyplusplus_wrapper
#define RenderedString_hpp__pyplusplus_wrapper

void register_RenderedString_class();

#endif//RenderedString_hpp__pyplusplus_wrapper
