// This file has been generated by Py++.

#ifndef TypedPropertyString_hpp__pyplusplus_wrapper
#define TypedPropertyString_hpp__pyplusplus_wrapper

void register_TypedPropertyString_class();

#endif//TypedPropertyString_hpp__pyplusplus_wrapper
