// This file has been generated by Py++.

#ifndef SingletonGlobalEventSet_hpp__pyplusplus_wrapper
#define SingletonGlobalEventSet_hpp__pyplusplus_wrapper

void register_SingletonGlobalEventSet_class();

#endif//SingletonGlobalEventSet_hpp__pyplusplus_wrapper
