// This file has been generated by Py++.

#ifndef WidgetComponentMap_hpp__pyplusplus_wrapper
#define WidgetComponentMap_hpp__pyplusplus_wrapper

void register_WidgetComponentMap_class();

#endif//WidgetComponentMap_hpp__pyplusplus_wrapper
