// This file has been generated by Py++.

#ifndef FalagardMappingIterator_hpp__pyplusplus_wrapper
#define FalagardMappingIterator_hpp__pyplusplus_wrapper

void register_FalagardMappingIterator_class();

#endif//FalagardMappingIterator_hpp__pyplusplus_wrapper
