// This file has been generated by Py++.

#ifndef DisplayEventArgs_hpp__pyplusplus_wrapper
#define DisplayEventArgs_hpp__pyplusplus_wrapper

void register_DisplayEventArgs_class();

#endif//DisplayEventArgs_hpp__pyplusplus_wrapper
