// This file has been generated by Py++.

#ifndef Connection_hpp__pyplusplus_wrapper
#define Connection_hpp__pyplusplus_wrapper

void register_Connection_class();

#endif//Connection_hpp__pyplusplus_wrapper
