// This file has been generated by Py++.

#ifndef OgreResourceProvider_hpp__pyplusplus_wrapper
#define OgreResourceProvider_hpp__pyplusplus_wrapper

void register_OgreResourceProvider_class();

#endif//OgreResourceProvider_hpp__pyplusplus_wrapper
