// This file has been generated by Py++.

#ifndef TypedPropertyColourUBox_hpp__pyplusplus_wrapper
#define TypedPropertyColourUBox_hpp__pyplusplus_wrapper

void register_TypedPropertyColourUBox_class();

#endif//TypedPropertyColourUBox_hpp__pyplusplus_wrapper
