// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionVector2f_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionVector2f_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionVector2f_class();

#endif//PropertyLinkDefinitionVector2f_hpp__pyplusplus_wrapper
