// This file has been generated by Py++.

#ifndef WidgetLookFeelMap_hpp__pyplusplus_wrapper
#define WidgetLookFeelMap_hpp__pyplusplus_wrapper

void register_WidgetLookFeelMap_class();

#endif//WidgetLookFeelMap_hpp__pyplusplus_wrapper
