// This file has been generated by Py++.

#ifndef TextComponentVector_hpp__pyplusplus_wrapper
#define TextComponentVector_hpp__pyplusplus_wrapper

void register_TextComponentVector_class();

#endif//TextComponentVector_hpp__pyplusplus_wrapper
