// This file has been generated by Py++.

#ifndef ImageCodec_hpp__pyplusplus_wrapper
#define ImageCodec_hpp__pyplusplus_wrapper

void register_ImageCodec_class();

#endif//ImageCodec_hpp__pyplusplus_wrapper
