// This file has been generated by Py++.

#ifndef FalagardPropertyBaseFont_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseFont_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseFont_class();

#endif//FalagardPropertyBaseFont_hpp__pyplusplus_wrapper
