// This file has been generated by Py++.

#ifndef ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f_hpp__pyplusplus_wrapper
#define ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f_class();

#endif//ConstBaseIterator_e31fba8abb652b91e7a416a794506e4f_hpp__pyplusplus_wrapper
