// This file has been generated by Py++.

#ifndef PropertyDefinitionImage_hpp__pyplusplus_wrapper
#define PropertyDefinitionImage_hpp__pyplusplus_wrapper

void register_PropertyDefinitionImage_class();

#endif//PropertyDefinitionImage_hpp__pyplusplus_wrapper
