// This file has been generated by Py++.

#ifndef ImageIterator_hpp__pyplusplus_wrapper
#define ImageIterator_hpp__pyplusplus_wrapper

void register_ImageIterator_class();

#endif//ImageIterator_hpp__pyplusplus_wrapper
