// This file has been generated by Py++.

#ifndef LBItemList_hpp__pyplusplus_wrapper
#define LBItemList_hpp__pyplusplus_wrapper

void register_LBItemList_class();

#endif//LBItemList_hpp__pyplusplus_wrapper
