// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionRectf_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionRectf_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionRectf_class();

#endif//PropertyLinkDefinitionRectf_hpp__pyplusplus_wrapper
