// This file has been generated by Py++.

#ifndef DefaultLogger_hpp__pyplusplus_wrapper
#define DefaultLogger_hpp__pyplusplus_wrapper

void register_DefaultLogger_class();

#endif//DefaultLogger_hpp__pyplusplus_wrapper
