// This file has been generated by Py++.

#ifndef TextureTarget_hpp__pyplusplus_wrapper
#define TextureTarget_hpp__pyplusplus_wrapper

void register_TextureTarget_class();

#endif//TextureTarget_hpp__pyplusplus_wrapper
