// This file has been generated by Py++.

#ifndef TypedPropertyImage_hpp__pyplusplus_wrapper
#define TypedPropertyImage_hpp__pyplusplus_wrapper

void register_TypedPropertyImage_class();

#endif//TypedPropertyImage_hpp__pyplusplus_wrapper
