// This file has been generated by Py++.

#ifndef PropertyDefinitionVerticalFormatting_hpp__pyplusplus_wrapper
#define PropertyDefinitionVerticalFormatting_hpp__pyplusplus_wrapper

void register_PropertyDefinitionVerticalFormatting_class();

#endif//PropertyDefinitionVerticalFormatting_hpp__pyplusplus_wrapper
