// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "_UDim__value_traits.pypp.hpp"
#include "vector_less__CEGUI_scope_UDim__greater_.pypp.hpp"

namespace bp = boost::python;

void register_vector_less__CEGUI_scope_UDim__greater__class(){

    bp::class_< std::vector< CEGUI::UDim > >("vector_less__CEGUI_scope_UDim__greater_")    
        .def( bp::indexing::vector_suite< std::vector< CEGUI::UDim > >() );

}
