// This file has been generated by Py++.

#ifndef FalagardPropertyBaseRectf_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseRectf_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseRectf_class();

#endif//FalagardPropertyBaseRectf_hpp__pyplusplus_wrapper
