// This file has been generated by Py++.

#ifndef ImageryComponentIterator_hpp__pyplusplus_wrapper
#define ImageryComponentIterator_hpp__pyplusplus_wrapper

void register_ImageryComponentIterator_class();

#endif//ImageryComponentIterator_hpp__pyplusplus_wrapper
