// This file has been generated by Py++.

#ifndef WindowFactoryIterator_hpp__pyplusplus_wrapper
#define WindowFactoryIterator_hpp__pyplusplus_wrapper

void register_WindowFactoryIterator_class();

#endif//WindowFactoryIterator_hpp__pyplusplus_wrapper
