// This file has been generated by Py++.

#ifndef NamedAreaMap_hpp__pyplusplus_wrapper
#define NamedAreaMap_hpp__pyplusplus_wrapper

void register_NamedAreaMap_class();

#endif//NamedAreaMap_hpp__pyplusplus_wrapper
