// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "RenderedStringWidgetComponent.pypp.hpp"

namespace bp = boost::python;

struct RenderedStringWidgetComponent_wrapper : CEGUI::RenderedStringWidgetComponent, bp::wrapper< CEGUI::RenderedStringWidgetComponent > {

    RenderedStringWidgetComponent_wrapper(CEGUI::RenderedStringWidgetComponent const & arg )
    : CEGUI::RenderedStringWidgetComponent( arg )
      , bp::wrapper< CEGUI::RenderedStringWidgetComponent >(){
        // copy constructor
        
    }

    RenderedStringWidgetComponent_wrapper( )
    : CEGUI::RenderedStringWidgetComponent( )
      , bp::wrapper< CEGUI::RenderedStringWidgetComponent >(){
        // null constructor
    
    }

    RenderedStringWidgetComponent_wrapper(::CEGUI::String const & widget_name )
    : CEGUI::RenderedStringWidgetComponent( boost::ref(widget_name) )
      , bp::wrapper< CEGUI::RenderedStringWidgetComponent >(){
        // constructor
    
    }

    RenderedStringWidgetComponent_wrapper(::CEGUI::Window * widget )
    : CEGUI::RenderedStringWidgetComponent( boost::python::ptr(widget) )
      , bp::wrapper< CEGUI::RenderedStringWidgetComponent >(){
        // constructor
    
    }

    virtual bool canSplit(  ) const  {
        if( bp::override func_canSplit = this->get_override( "canSplit" ) )
            return func_canSplit(  );
        else{
            return this->CEGUI::RenderedStringWidgetComponent::canSplit(  );
        }
    }
    
    bool default_canSplit(  ) const  {
        return CEGUI::RenderedStringWidgetComponent::canSplit( );
    }

    virtual ::CEGUI::RenderedStringWidgetComponent * clone(  ) const  {
        if( bp::override func_clone = this->get_override( "clone" ) )
            return func_clone(  );
        else{
            return this->CEGUI::RenderedStringWidgetComponent::clone(  );
        }
    }
    
    ::CEGUI::RenderedStringWidgetComponent * default_clone(  ) const  {
        return CEGUI::RenderedStringWidgetComponent::clone( );
    }

    virtual void draw( ::CEGUI::Window const * ref_wnd, ::CEGUI::GeometryBuffer & buffer, ::CEGUI::Vector2f const & position, ::CEGUI::ColourRect const * mod_colours, ::CEGUI::Rectf const * clip_rect, float const vertical_space, float const space_extra ) const  {
        if( bp::override func_draw = this->get_override( "draw" ) )
            func_draw( boost::python::ptr(ref_wnd), boost::ref(buffer), boost::ref(position), boost::python::ptr(mod_colours), boost::python::ptr(clip_rect), vertical_space, space_extra );
        else{
            this->CEGUI::RenderedStringWidgetComponent::draw( boost::python::ptr(ref_wnd), boost::ref(buffer), boost::ref(position), boost::python::ptr(mod_colours), boost::python::ptr(clip_rect), vertical_space, space_extra );
        }
    }
    
    void default_draw( ::CEGUI::Window const * ref_wnd, ::CEGUI::GeometryBuffer & buffer, ::CEGUI::Vector2f const & position, ::CEGUI::ColourRect const * mod_colours, ::CEGUI::Rectf const * clip_rect, float const vertical_space, float const space_extra ) const  {
        CEGUI::RenderedStringWidgetComponent::draw( boost::python::ptr(ref_wnd), boost::ref(buffer), boost::ref(position), boost::python::ptr(mod_colours), boost::python::ptr(clip_rect), vertical_space, space_extra );
    }

    ::CEGUI::Window * getEffectiveWindow( ::CEGUI::Window const * ref_wnd ) const {
        return CEGUI::RenderedStringWidgetComponent::getEffectiveWindow( boost::python::ptr(ref_wnd) );
    }

    virtual ::CEGUI::Sizef getPixelSize( ::CEGUI::Window const * ref_wnd ) const  {
        if( bp::override func_getPixelSize = this->get_override( "getPixelSize" ) )
            return func_getPixelSize( boost::python::ptr(ref_wnd) );
        else{
            return this->CEGUI::RenderedStringWidgetComponent::getPixelSize( boost::python::ptr(ref_wnd) );
        }
    }
    
    ::CEGUI::Sizef default_getPixelSize( ::CEGUI::Window const * ref_wnd ) const  {
        return CEGUI::RenderedStringWidgetComponent::getPixelSize( boost::python::ptr(ref_wnd) );
    }

    virtual ::size_t getSpaceCount(  ) const  {
        if( bp::override func_getSpaceCount = this->get_override( "getSpaceCount" ) )
            return func_getSpaceCount(  );
        else{
            return this->CEGUI::RenderedStringWidgetComponent::getSpaceCount(  );
        }
    }
    
    ::size_t default_getSpaceCount(  ) const  {
        return CEGUI::RenderedStringWidgetComponent::getSpaceCount( );
    }

    virtual void setSelection( ::CEGUI::Window const * ref_wnd, float const start, float const end ) {
        if( bp::override func_setSelection = this->get_override( "setSelection" ) )
            func_setSelection( boost::python::ptr(ref_wnd), start, end );
        else{
            this->CEGUI::RenderedStringWidgetComponent::setSelection( boost::python::ptr(ref_wnd), start, end );
        }
    }
    
    void default_setSelection( ::CEGUI::Window const * ref_wnd, float const start, float const end ) {
        CEGUI::RenderedStringWidgetComponent::setSelection( boost::python::ptr(ref_wnd), start, end );
    }

    virtual ::CEGUI::RenderedStringWidgetComponent * split( ::CEGUI::Window const * ref_wnd, float split_point, bool first_component ) {
        if( bp::override func_split = this->get_override( "split" ) )
            return func_split( boost::python::ptr(ref_wnd), split_point, first_component );
        else{
            return this->CEGUI::RenderedStringWidgetComponent::split( boost::python::ptr(ref_wnd), split_point, first_component );
        }
    }
    
    ::CEGUI::RenderedStringWidgetComponent * default_split( ::CEGUI::Window const * ref_wnd, float split_point, bool first_component ) {
        return CEGUI::RenderedStringWidgetComponent::split( boost::python::ptr(ref_wnd), split_point, first_component );
    }

};

void register_RenderedStringWidgetComponent_class(){

    { //::CEGUI::RenderedStringWidgetComponent
        typedef bp::class_< RenderedStringWidgetComponent_wrapper, bp::bases< CEGUI::RenderedStringComponent > > RenderedStringWidgetComponent_exposer_t;
        RenderedStringWidgetComponent_exposer_t RenderedStringWidgetComponent_exposer = RenderedStringWidgetComponent_exposer_t( "RenderedStringWidgetComponent", "! String component that moves a widget to appear as part of the string.\n", bp::init< >("! Constructor\n") );
        bp::scope RenderedStringWidgetComponent_scope( RenderedStringWidgetComponent_exposer );
        RenderedStringWidgetComponent_exposer.def( bp::init< CEGUI::String const & >(( bp::arg("widget_name") ), "! Constructor\n") );
        bp::implicitly_convertible< CEGUI::String const &, CEGUI::RenderedStringWidgetComponent >();
        RenderedStringWidgetComponent_exposer.def( bp::init< CEGUI::Window * >(( bp::arg("widget") )) );
        bp::implicitly_convertible< CEGUI::Window *, CEGUI::RenderedStringWidgetComponent >();
        { //::CEGUI::RenderedStringWidgetComponent::canSplit
        
            typedef bool ( ::CEGUI::RenderedStringWidgetComponent::*canSplit_function_type )(  ) const;
            typedef bool ( RenderedStringWidgetComponent_wrapper::*default_canSplit_function_type )(  ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "canSplit"
                , canSplit_function_type(&::CEGUI::RenderedStringWidgetComponent::canSplit)
                , default_canSplit_function_type(&RenderedStringWidgetComponent_wrapper::default_canSplit) );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::clone
        
            typedef ::CEGUI::RenderedStringWidgetComponent * ( ::CEGUI::RenderedStringWidgetComponent::*clone_function_type )(  ) const;
            typedef ::CEGUI::RenderedStringWidgetComponent * ( RenderedStringWidgetComponent_wrapper::*default_clone_function_type )(  ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "clone"
                , clone_function_type(&::CEGUI::RenderedStringWidgetComponent::clone)
                , default_clone_function_type(&RenderedStringWidgetComponent_wrapper::default_clone)
                , bp::return_value_policy< bp::reference_existing_object >() );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::draw
        
            typedef void ( ::CEGUI::RenderedStringWidgetComponent::*draw_function_type )( ::CEGUI::Window const *,::CEGUI::GeometryBuffer &,::CEGUI::Vector2f const &,::CEGUI::ColourRect const *,::CEGUI::Rectf const *,float const,float const ) const;
            typedef void ( RenderedStringWidgetComponent_wrapper::*default_draw_function_type )( ::CEGUI::Window const *,::CEGUI::GeometryBuffer &,::CEGUI::Vector2f const &,::CEGUI::ColourRect const *,::CEGUI::Rectf const *,float const,float const ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "draw"
                , draw_function_type(&::CEGUI::RenderedStringWidgetComponent::draw)
                , default_draw_function_type(&RenderedStringWidgetComponent_wrapper::default_draw)
                , ( bp::arg("ref_wnd"), bp::arg("buffer"), bp::arg("position"), bp::arg("mod_colours"), bp::arg("clip_rect"), bp::arg("vertical_space"), bp::arg("space_extra") ) );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::getEffectiveWindow
        
            typedef ::CEGUI::Window * ( RenderedStringWidgetComponent_wrapper::*getEffectiveWindow_function_type )( ::CEGUI::Window const * ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "getEffectiveWindow"
                , getEffectiveWindow_function_type( &RenderedStringWidgetComponent_wrapper::getEffectiveWindow )
                , ( bp::arg("ref_wnd") )
                , bp::return_value_policy< bp::reference_existing_object >() );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::getPixelSize
        
            typedef ::CEGUI::Sizef ( ::CEGUI::RenderedStringWidgetComponent::*getPixelSize_function_type )( ::CEGUI::Window const * ) const;
            typedef ::CEGUI::Sizef ( RenderedStringWidgetComponent_wrapper::*default_getPixelSize_function_type )( ::CEGUI::Window const * ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "getPixelSize"
                , getPixelSize_function_type(&::CEGUI::RenderedStringWidgetComponent::getPixelSize)
                , default_getPixelSize_function_type(&RenderedStringWidgetComponent_wrapper::default_getPixelSize)
                , ( bp::arg("ref_wnd") ) );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::getSpaceCount
        
            typedef ::size_t ( ::CEGUI::RenderedStringWidgetComponent::*getSpaceCount_function_type )(  ) const;
            typedef ::size_t ( RenderedStringWidgetComponent_wrapper::*default_getSpaceCount_function_type )(  ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "getSpaceCount"
                , getSpaceCount_function_type(&::CEGUI::RenderedStringWidgetComponent::getSpaceCount)
                , default_getSpaceCount_function_type(&RenderedStringWidgetComponent_wrapper::default_getSpaceCount) );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::getWindow
        
            typedef ::CEGUI::Window const * ( ::CEGUI::RenderedStringWidgetComponent::*getWindow_function_type )(  ) const;
            
            RenderedStringWidgetComponent_exposer.def( 
                "getWindow"
                , getWindow_function_type( &::CEGUI::RenderedStringWidgetComponent::getWindow )
                , bp::return_value_policy< bp::reference_existing_object >()
                , "! Set the window to be controlled by this component.\n\
            ! return the window currently controlled by this component\n" );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::setSelection
        
            typedef void ( ::CEGUI::RenderedStringWidgetComponent::*setSelection_function_type )( ::CEGUI::Window const *,float const,float const ) ;
            typedef void ( RenderedStringWidgetComponent_wrapper::*default_setSelection_function_type )( ::CEGUI::Window const *,float const,float const ) ;
            
            RenderedStringWidgetComponent_exposer.def( 
                "setSelection"
                , setSelection_function_type(&::CEGUI::RenderedStringWidgetComponent::setSelection)
                , default_setSelection_function_type(&RenderedStringWidgetComponent_wrapper::default_setSelection)
                , ( bp::arg("ref_wnd"), bp::arg("start"), bp::arg("end") ) );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::setWindow
        
            typedef void ( ::CEGUI::RenderedStringWidgetComponent::*setWindow_function_type )( ::CEGUI::String const & ) ;
            
            RenderedStringWidgetComponent_exposer.def( 
                "setWindow"
                , setWindow_function_type( &::CEGUI::RenderedStringWidgetComponent::setWindow )
                , ( bp::arg("widget_name") )
                , "! Set the window to be controlled by this component.\n" );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::setWindow
        
            typedef void ( ::CEGUI::RenderedStringWidgetComponent::*setWindow_function_type )( ::CEGUI::Window * ) ;
            
            RenderedStringWidgetComponent_exposer.def( 
                "setWindow"
                , setWindow_function_type( &::CEGUI::RenderedStringWidgetComponent::setWindow )
                , ( bp::arg("widget") )
                , "! Set the window to be controlled by this component.\n\
            ! Set the window to be controlled by this component.\n" );
        
        }
        { //::CEGUI::RenderedStringWidgetComponent::split
        
            typedef ::CEGUI::RenderedStringWidgetComponent * ( ::CEGUI::RenderedStringWidgetComponent::*split_function_type )( ::CEGUI::Window const *,float,bool ) ;
            typedef ::CEGUI::RenderedStringWidgetComponent * ( RenderedStringWidgetComponent_wrapper::*default_split_function_type )( ::CEGUI::Window const *,float,bool ) ;
            
            RenderedStringWidgetComponent_exposer.def( 
                "split"
                , split_function_type(&::CEGUI::RenderedStringWidgetComponent::split)
                , default_split_function_type(&RenderedStringWidgetComponent_wrapper::default_split)
                , ( bp::arg("ref_wnd"), bp::arg("split_point"), bp::arg("first_component") )
                , bp::return_value_policy< bp::reference_existing_object >() );
        
        }
    }

}
