// This file has been generated by Py++.

#ifndef PropertyDefinitionFloat_hpp__pyplusplus_wrapper
#define PropertyDefinitionFloat_hpp__pyplusplus_wrapper

void register_PropertyDefinitionFloat_class();

#endif//PropertyDefinitionFloat_hpp__pyplusplus_wrapper
