// This file has been generated by Py++.

#ifndef ItemEntry_hpp__pyplusplus_wrapper
#define ItemEntry_hpp__pyplusplus_wrapper

void register_ItemEntry_class();

#endif//ItemEntry_hpp__pyplusplus_wrapper
