// This file has been generated by Py++.

#ifndef StdPairCEGUIImageImageFactory_hpp__pyplusplus_wrapper
#define StdPairCEGUIImageImageFactory_hpp__pyplusplus_wrapper

void register_StdPairCEGUIImageImageFactory_class();

#endif//StdPairCEGUIImageImageFactory_hpp__pyplusplus_wrapper
