// This file has been generated by Py++.

#ifndef TextComponentIterator_hpp__pyplusplus_wrapper
#define TextComponentIterator_hpp__pyplusplus_wrapper

void register_TextComponentIterator_class();

#endif//TextComponentIterator_hpp__pyplusplus_wrapper
