// This file has been generated by Py++.

#ifndef XMLHandler_hpp__pyplusplus_wrapper
#define XMLHandler_hpp__pyplusplus_wrapper

void register_XMLHandler_class();

#endif//XMLHandler_hpp__pyplusplus_wrapper
