// This file has been generated by Py++.

#ifndef SingletonRenderEffectManager_hpp__pyplusplus_wrapper
#define SingletonRenderEffectManager_hpp__pyplusplus_wrapper

void register_SingletonRenderEffectManager_class();

#endif//SingletonRenderEffectManager_hpp__pyplusplus_wrapper
