// This file has been generated by Py++.

#ifndef TypedPropertyColourUDim_hpp__pyplusplus_wrapper
#define TypedPropertyColourUDim_hpp__pyplusplus_wrapper

void register_TypedPropertyColourUDim_class();

#endif//TypedPropertyColourUDim_hpp__pyplusplus_wrapper
