// This file has been generated by Py++.

#ifndef Animation_hpp__pyplusplus_wrapper
#define Animation_hpp__pyplusplus_wrapper

void register_Animation_class();

#endif//Animation_hpp__pyplusplus_wrapper
