// This file has been generated by Py++.

#ifndef NamedElementEventArgs_hpp__pyplusplus_wrapper
#define NamedElementEventArgs_hpp__pyplusplus_wrapper

void register_NamedElementEventArgs_class();

#endif//NamedElementEventArgs_hpp__pyplusplus_wrapper
