// This file has been generated by Py++.

#ifndef FontIterator_hpp__pyplusplus_wrapper
#define FontIterator_hpp__pyplusplus_wrapper

void register_FontIterator_class();

#endif//FontIterator_hpp__pyplusplus_wrapper
