// This file has been generated by Py++.

#ifndef OpenGLTextureTarget_hpp__pyplusplus_wrapper
#define OpenGLTextureTarget_hpp__pyplusplus_wrapper

void register_OpenGLTextureTarget_class();

#endif//OpenGLTextureTarget_hpp__pyplusplus_wrapper
