// This file has been generated by Py++.

#ifndef TypedPropertyRectf_hpp__pyplusplus_wrapper
#define TypedPropertyRectf_hpp__pyplusplus_wrapper

void register_TypedPropertyRectf_class();

#endif//TypedPropertyRectf_hpp__pyplusplus_wrapper
