// This file has been generated by Py++.

#ifndef Scrollbar_hpp__pyplusplus_wrapper
#define Scrollbar_hpp__pyplusplus_wrapper

void register_Scrollbar_class();

#endif//Scrollbar_hpp__pyplusplus_wrapper
