// This file has been generated by Py++.

#ifndef PropertyDefinitionIterator_hpp__pyplusplus_wrapper
#define PropertyDefinitionIterator_hpp__pyplusplus_wrapper

void register_PropertyDefinitionIterator_class();

#endif//PropertyDefinitionIterator_hpp__pyplusplus_wrapper
