// This file has been generated by Py++.

#ifndef PropertyDefinitionString_hpp__pyplusplus_wrapper
#define PropertyDefinitionString_hpp__pyplusplus_wrapper

void register_PropertyDefinitionString_class();

#endif//PropertyDefinitionString_hpp__pyplusplus_wrapper
