// This file has been generated by Py++.

#ifndef WidgetLookFeel_hpp__pyplusplus_wrapper
#define WidgetLookFeel_hpp__pyplusplus_wrapper

void register_WidgetLookFeel_class();

#endif//WidgetLookFeel_hpp__pyplusplus_wrapper
