// This file has been generated by Py++.

#ifndef ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133_hpp__pyplusplus_wrapper
#define ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133_class();

#endif//ConstBaseIterator_23c5f09e76c89ed46f1671ac87f5d133_hpp__pyplusplus_wrapper
