// This file has been generated by Py++.

#ifndef RadioButton_hpp__pyplusplus_wrapper
#define RadioButton_hpp__pyplusplus_wrapper

void register_RadioButton_class();

#endif//RadioButton_hpp__pyplusplus_wrapper
