// This file has been generated by Py++.

#ifndef SingletonWindowFactoryManager_hpp__pyplusplus_wrapper
#define SingletonWindowFactoryManager_hpp__pyplusplus_wrapper

void register_SingletonWindowFactoryManager_class();

#endif//SingletonWindowFactoryManager_hpp__pyplusplus_wrapper
