// This file has been generated by Py++.

#ifndef SectionSpecification_hpp__pyplusplus_wrapper
#define SectionSpecification_hpp__pyplusplus_wrapper

void register_SectionSpecification_class();

#endif//SectionSpecification_hpp__pyplusplus_wrapper
