// This file has been generated by Py++.

#ifndef ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1_hpp__pyplusplus_wrapper
#define ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1_class();

#endif//ConstBaseIterator_663c592fa58613dab4f9509d04eb6cc1_hpp__pyplusplus_wrapper
