// This file has been generated by Py++.

#ifndef SingletonSchemeManager_hpp__pyplusplus_wrapper
#define SingletonSchemeManager_hpp__pyplusplus_wrapper

void register_SingletonSchemeManager_class();

#endif//SingletonSchemeManager_hpp__pyplusplus_wrapper
