// This file has been generated by Py++.

#ifndef TargetTypeStack_hpp__pyplusplus_wrapper
#define TargetTypeStack_hpp__pyplusplus_wrapper

void register_TargetTypeStack_class();

#endif//TargetTypeStack_hpp__pyplusplus_wrapper
