// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "ComponentList.pypp.hpp"

namespace bp = boost::python;

void register_ComponentList_class(){

    { //::std::vector< CEGUI::RenderedStringComponent* >
        typedef bp::class_< std::vector< CEGUI::RenderedStringComponent* > > ComponentList_exposer_t;
        ComponentList_exposer_t ComponentList_exposer = ComponentList_exposer_t( "ComponentList" );
        bp::scope ComponentList_scope( ComponentList_exposer );
        ComponentList_exposer.def( bp::indexing::vector_suite< std::vector< CEGUI::RenderedStringComponent* > >::with_policies(bp::return_internal_reference< >()) );
    }

}
