// This file has been generated by Py++.

#ifndef FontEventArgs_hpp__pyplusplus_wrapper
#define FontEventArgs_hpp__pyplusplus_wrapper

void register_FontEventArgs_class();

#endif//FontEventArgs_hpp__pyplusplus_wrapper
