// This file has been generated by Py++.

#ifndef ComboDropList_hpp__pyplusplus_wrapper
#define ComboDropList_hpp__pyplusplus_wrapper

void register_ComboDropList_class();

#endif//ComboDropList_hpp__pyplusplus_wrapper
