// This file has been generated by Py++.

#ifndef PropertyDefinitionColourUBox_hpp__pyplusplus_wrapper
#define PropertyDefinitionColourUBox_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColourUBox_class();

#endif//PropertyDefinitionColourUBox_hpp__pyplusplus_wrapper
