// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "TextComponentVector.pypp.hpp"

namespace bp = boost::python;

void register_TextComponentVector_class(){

    bp::class_< std::vector< CEGUI::TextComponent* > >("TextComponentVector")    
        .def( bp::indexing::vector_suite< std::vector< CEGUI::TextComponent* > >::with_policies(bp::return_internal_reference< >()) );

}
