// This file has been generated by Py++.

#ifndef PropertyDefinitionUint_hpp__pyplusplus_wrapper
#define PropertyDefinitionUint_hpp__pyplusplus_wrapper

void register_PropertyDefinitionUint_class();

#endif//PropertyDefinitionUint_hpp__pyplusplus_wrapper
