// This file has been generated by Py++.

#ifndef OpenGLGeometryBufferBase_hpp__pyplusplus_wrapper
#define OpenGLGeometryBufferBase_hpp__pyplusplus_wrapper

void register_OpenGLGeometryBufferBase_class();

#endif//OpenGLGeometryBufferBase_hpp__pyplusplus_wrapper
