// This file has been generated by Py++.

#ifndef SchemeManager_hpp__pyplusplus_wrapper
#define SchemeManager_hpp__pyplusplus_wrapper

void register_SchemeManager_class();

#endif//SchemeManager_hpp__pyplusplus_wrapper
