// This file has been generated by Py++.

#ifndef LayerSpecificationVector_hpp__pyplusplus_wrapper
#define LayerSpecificationVector_hpp__pyplusplus_wrapper

void register_LayerSpecificationVector_class();

#endif//LayerSpecificationVector_hpp__pyplusplus_wrapper
