// This file has been generated by Py++.

#ifndef ImageryComponent_hpp__pyplusplus_wrapper
#define ImageryComponent_hpp__pyplusplus_wrapper

void register_ImageryComponent_class();

#endif//ImageryComponent_hpp__pyplusplus_wrapper
