#include "CEGUI/String.h"
#include "CEGUI/GeometryBuffer.h"
#include "CEGUI/RenderEffect.h"
#include "CEGUI/Texture.h"
#include "CEGUI/TextureTarget.h"
#include "CEGUI/GUIContext.h"
#include "CEGUI/Vertex.h"

#include "CEGUI/RendererModules/OpenGL/GLRenderer.h"

#include "CEGUI/RendererModules/OpenGL/ViewportTarget.h"
#include "CEGUI/RendererModules/OpenGL/TextureTarget.h"
#include "CEGUI/RendererModules/OpenGL/GeometryBufferBase.h"

#include "CEGUI/RendererModules/OpenGL/GL3Renderer.h"
#include "CEGUI/RendererModules/OpenGL/Shader.h"
#include "CEGUI/RendererModules/OpenGL/StateChangeWrapper.h"
