// This file has been generated by Py++.

#ifndef GUIContext_hpp__pyplusplus_wrapper
#define GUIContext_hpp__pyplusplus_wrapper

void register_GUIContext_class();

#endif//GUIContext_hpp__pyplusplus_wrapper
