// This file has been generated by Py++.

#ifndef ColourRect_hpp__pyplusplus_wrapper
#define ColourRect_hpp__pyplusplus_wrapper

void register_ColourRect_class();

#endif//ColourRect_hpp__pyplusplus_wrapper
