// This file has been generated by Py++.

#ifndef WidgetComponent_hpp__pyplusplus_wrapper
#define WidgetComponent_hpp__pyplusplus_wrapper

void register_WidgetComponent_class();

#endif//WidgetComponent_hpp__pyplusplus_wrapper
