// This file has been generated by Py++.

#ifndef GUIContextEventArgs_hpp__pyplusplus_wrapper
#define GUIContextEventArgs_hpp__pyplusplus_wrapper

void register_GUIContextEventArgs_class();

#endif//GUIContextEventArgs_hpp__pyplusplus_wrapper
