// This file has been generated by Py++.

#ifndef ImageryIterator_hpp__pyplusplus_wrapper
#define ImageryIterator_hpp__pyplusplus_wrapper

void register_ImageryIterator_class();

#endif//ImageryIterator_hpp__pyplusplus_wrapper
