// This file has been generated by Py++.

#ifndef BoundSlot_hpp__pyplusplus_wrapper
#define BoundSlot_hpp__pyplusplus_wrapper

void register_BoundSlot_class();

#endif//BoundSlot_hpp__pyplusplus_wrapper
