// This file has been generated by Py++.

#ifndef OpenGLViewportTarget_hpp__pyplusplus_wrapper
#define OpenGLViewportTarget_hpp__pyplusplus_wrapper

void register_OpenGLViewportTarget_class();

#endif//OpenGLViewportTarget_hpp__pyplusplus_wrapper
