// This file has been generated by Py++.

#ifndef NullRenderer_hpp__pyplusplus_wrapper
#define NullRenderer_hpp__pyplusplus_wrapper

void register_NullRenderer_class();

#endif//NullRenderer_hpp__pyplusplus_wrapper
