// This file has been generated by Py++.

#ifndef RenderEffect_hpp__pyplusplus_wrapper
#define RenderEffect_hpp__pyplusplus_wrapper

void register_RenderEffect_class();

#endif//RenderEffect_hpp__pyplusplus_wrapper
