// This file has been generated by Py++.

#ifndef MouseEventArgs_hpp__pyplusplus_wrapper
#define MouseEventArgs_hpp__pyplusplus_wrapper

void register_MouseEventArgs_class();

#endif//MouseEventArgs_hpp__pyplusplus_wrapper
