// This file has been generated by Py++.

#ifndef RegexMatchStateEventArgs_hpp__pyplusplus_wrapper
#define RegexMatchStateEventArgs_hpp__pyplusplus_wrapper

void register_RegexMatchStateEventArgs_class();

#endif//RegexMatchStateEventArgs_hpp__pyplusplus_wrapper
