// This file has been generated by Py++.

#ifndef FalagardPropertyBaseColourUDim_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseColourUDim_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseColourUDim_class();

#endif//FalagardPropertyBaseColourUDim_hpp__pyplusplus_wrapper
