// This file has been generated by Py++.

#ifndef FalagardPropertyBaseImage_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseImage_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseImage_class();

#endif//FalagardPropertyBaseImage_hpp__pyplusplus_wrapper
