// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/map.hpp"
#include "generators/include/python_CEGUI.h"
#include "NamedAreaMap.pypp.hpp"

namespace bp = boost::python;

void register_NamedAreaMap_class(){

    { //::std::map<CEGUI::String, CEGUI::NamedArea*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::NamedArea*> > >
        typedef bp::class_< std::map<CEGUI::String, CEGUI::NamedArea*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::NamedArea*> > > > NamedAreaMap_exposer_t;
        NamedAreaMap_exposer_t NamedAreaMap_exposer = NamedAreaMap_exposer_t( "NamedAreaMap" );
        bp::scope NamedAreaMap_scope( NamedAreaMap_exposer );
        NamedAreaMap_exposer.def( bp::indexing::map_suite< std::map<CEGUI::String, CEGUI::NamedArea*, CEGUI::StringFastLessCompare, std::allocator<std::pair<CEGUI::String const, CEGUI::NamedArea*> > > >() );
    }

}
