// This file has been generated by Py++.

#ifndef EventIterator_hpp__pyplusplus_wrapper
#define EventIterator_hpp__pyplusplus_wrapper

void register_EventIterator_class();

#endif//EventIterator_hpp__pyplusplus_wrapper
