// This file has been generated by Py++.

#ifndef TypedPropertyHorizontalTextFormatting_hpp__pyplusplus_wrapper
#define TypedPropertyHorizontalTextFormatting_hpp__pyplusplus_wrapper

void register_TypedPropertyHorizontalTextFormatting_class();

#endif//TypedPropertyHorizontalTextFormatting_hpp__pyplusplus_wrapper
