// This file has been generated by Py++.

#ifndef TextUtils_hpp__pyplusplus_wrapper
#define TextUtils_hpp__pyplusplus_wrapper

void register_TextUtils_class();

#endif//TextUtils_hpp__pyplusplus_wrapper
