// This file has been generated by Py++.

#ifndef WindowFactory_hpp__pyplusplus_wrapper
#define WindowFactory_hpp__pyplusplus_wrapper

void register_WindowFactory_class();

#endif//WindowFactory_hpp__pyplusplus_wrapper
