// This file has been generated by Py++.

#ifndef MouseCursor_hpp__pyplusplus_wrapper
#define MouseCursor_hpp__pyplusplus_wrapper

void register_MouseCursor_class();

#endif//MouseCursor_hpp__pyplusplus_wrapper
