// This file has been generated by Py++.

#ifndef VerticalLayoutContainer_hpp__pyplusplus_wrapper
#define VerticalLayoutContainer_hpp__pyplusplus_wrapper

void register_VerticalLayoutContainer_class();

#endif//VerticalLayoutContainer_hpp__pyplusplus_wrapper
