// This file has been generated by Py++.

#ifndef ComponentArea_hpp__pyplusplus_wrapper
#define ComponentArea_hpp__pyplusplus_wrapper

void register_ComponentArea_class();

#endif//ComponentArea_hpp__pyplusplus_wrapper
