// This file has been generated by Py++.

#ifndef SingletonSystem_hpp__pyplusplus_wrapper
#define SingletonSystem_hpp__pyplusplus_wrapper

void register_SingletonSystem_class();

#endif//SingletonSystem_hpp__pyplusplus_wrapper
