// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionFont_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionFont_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionFont_class();

#endif//PropertyLinkDefinitionFont_hpp__pyplusplus_wrapper
