// This file has been generated by Py++.

#ifndef AnimationNameIterator_hpp__pyplusplus_wrapper
#define AnimationNameIterator_hpp__pyplusplus_wrapper

void register_AnimationNameIterator_class();

#endif//AnimationNameIterator_hpp__pyplusplus_wrapper
