// This file has been generated by Py++.

#ifndef Exception_hpp__pyplusplus_wrapper
#define Exception_hpp__pyplusplus_wrapper

void register_Exception_class();

#endif//Exception_hpp__pyplusplus_wrapper
