// This file has been generated by Py++.

#ifndef StdPairCEGUIStringString_hpp__pyplusplus_wrapper
#define StdPairCEGUIStringString_hpp__pyplusplus_wrapper

void register_StdPairCEGUIStringString_class();

#endif//StdPairCEGUIStringString_hpp__pyplusplus_wrapper
