// This file has been generated by Py++.

#ifndef FalagardPropertyBaseVerticalFormatting_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseVerticalFormatting_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseVerticalFormatting_class();

#endif//FalagardPropertyBaseVerticalFormatting_hpp__pyplusplus_wrapper
