// This file has been generated by Py++.

#ifndef FrameComponent_hpp__pyplusplus_wrapper
#define FrameComponent_hpp__pyplusplus_wrapper

void register_FrameComponent_class();

#endif//FrameComponent_hpp__pyplusplus_wrapper
