// This file has been generated by Py++.

#ifndef PropertyLinkDefinitionColourUVector2_hpp__pyplusplus_wrapper
#define PropertyLinkDefinitionColourUVector2_hpp__pyplusplus_wrapper

void register_PropertyLinkDefinitionColourUVector2_class();

#endif//PropertyLinkDefinitionColourUVector2_hpp__pyplusplus_wrapper
