// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "LBItemList.pypp.hpp"

namespace bp = boost::python;

void register_LBItemList_class(){

    { //::std::vector< CEGUI::TreeItem* >
        typedef bp::class_< std::vector< CEGUI::TreeItem* > > LBItemList_exposer_t;
        LBItemList_exposer_t LBItemList_exposer = LBItemList_exposer_t( "LBItemList" );
        bp::scope LBItemList_scope( LBItemList_exposer );
        LBItemList_exposer.def( bp::indexing::vector_suite< std::vector< CEGUI::TreeItem* > >::with_policies(bp::return_internal_reference< >()) );
    }

}
