// This file has been generated by Py++.

#ifndef ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c_hpp__pyplusplus_wrapper
#define ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c_class();

#endif//ConstBaseIterator_45e2d4f27e29c3a5b74179f6cfabbc4c_hpp__pyplusplus_wrapper
