// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "Subscriber.pypp.hpp"

namespace bp = boost::python;

void register_Subscriber_class(){

    { //::CEGUI::SubscriberSlot
        typedef bp::class_< CEGUI::SubscriberSlot > Subscriber_exposer_t;
        Subscriber_exposer_t Subscriber_exposer = Subscriber_exposer_t( "Subscriber", bp::init< >("*!\n\
        \n\
            Default constructor.  Creates a SubscriberSlot with no bound slot.\n\
        *\n") );
        bp::scope Subscriber_scope( Subscriber_exposer );
        { //::CEGUI::SubscriberSlot::cleanup
        
            typedef void ( ::CEGUI::SubscriberSlot::*cleanup_function_type )(  ) ;
            
            Subscriber_exposer.def( 
                "cleanup"
                , cleanup_function_type( &::CEGUI::SubscriberSlot::cleanup )
                , "*!\n\
            \n\
                Disconnects the slot internally and performs any required cleanup\n\
                operations.\n\
            *\n" );
        
        }
        { //::CEGUI::SubscriberSlot::connected
        
            typedef bool ( ::CEGUI::SubscriberSlot::*connected_function_type )(  ) const;
            
            Subscriber_exposer.def( 
                "connected"
                , connected_function_type( &::CEGUI::SubscriberSlot::connected )
                , "*!\n\
            \n\
                Returns whether the SubscriberSlot is internally connected (bound).\n\
            *\n" );
        
        }
        { //::CEGUI::SubscriberSlot::operator()
        
            typedef bool ( ::CEGUI::SubscriberSlot::*__call___function_type )( ::CEGUI::EventArgs const & ) const;
            
            Subscriber_exposer.def( 
                "__call__"
                , __call___function_type( &::CEGUI::SubscriberSlot::operator() )
                , ( bp::arg("args") )
                , "*!\n\
            \n\
                Invokes the slot functor that is bound to this Subscriber.  Returns\n\
                whatever the slot returns, unless there is not slot bound when false is\n\
                always returned.\n\
            *\n" );
        
        }
    }

}
