// This file has been generated by Py++.

#ifndef RenderQueueEventArgs_hpp__pyplusplus_wrapper
#define RenderQueueEventArgs_hpp__pyplusplus_wrapper

void register_RenderQueueEventArgs_class();

#endif//RenderQueueEventArgs_hpp__pyplusplus_wrapper
