// This file has been generated by Py++.

#ifndef DefaultResourceProvider_hpp__pyplusplus_wrapper
#define DefaultResourceProvider_hpp__pyplusplus_wrapper

void register_DefaultResourceProvider_class();

#endif//DefaultResourceProvider_hpp__pyplusplus_wrapper
