// This file has been generated by Py++.

#ifndef PropertyReceiver_hpp__pyplusplus_wrapper
#define PropertyReceiver_hpp__pyplusplus_wrapper

void register_PropertyReceiver_class();

#endif//PropertyReceiver_hpp__pyplusplus_wrapper
