// This file has been generated by Py++.

#ifndef ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923_hpp__pyplusplus_wrapper
#define ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923_class();

#endif//ConstBaseIterator_3dfe55a685a628d9556dccd6ce85e923_hpp__pyplusplus_wrapper
