// This file has been generated by Py++.

#ifndef WidgetLookIterator_hpp__pyplusplus_wrapper
#define WidgetLookIterator_hpp__pyplusplus_wrapper

void register_WidgetLookIterator_class();

#endif//WidgetLookIterator_hpp__pyplusplus_wrapper
