// This file has been generated by Py++.

#ifndef RenderEffectFactory_hpp__pyplusplus_wrapper
#define RenderEffectFactory_hpp__pyplusplus_wrapper

void register_RenderEffectFactory_class();

#endif//RenderEffectFactory_hpp__pyplusplus_wrapper
