// This file has been generated by Py++.

#ifndef SingletonAnimationManager_hpp__pyplusplus_wrapper
#define SingletonAnimationManager_hpp__pyplusplus_wrapper

void register_SingletonAnimationManager_class();

#endif//SingletonAnimationManager_hpp__pyplusplus_wrapper
