// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "RawDataContainer.pypp.hpp"

namespace bp = boost::python;

void register_RawDataContainer_class(){

    { //::CEGUI::RawDataContainer
        typedef bp::class_< CEGUI::RawDataContainer > RawDataContainer_exposer_t;
        RawDataContainer_exposer_t RawDataContainer_exposer = RawDataContainer_exposer_t( "RawDataContainer", bp::init< >("*************************************************************************\n\
           Construction and Destruction\n\
        *************************************************************************\n\
        *!\n\
        \n\
           Constructor for RawDataContainer class\n\
        *\n") );
        bp::scope RawDataContainer_scope( RawDataContainer_exposer );
    }

}
