// This file has been generated by Py++.

#ifndef StateImagery_hpp__pyplusplus_wrapper
#define StateImagery_hpp__pyplusplus_wrapper

void register_StateImagery_class();

#endif//StateImagery_hpp__pyplusplus_wrapper
