// This file has been generated by Py++.

#ifndef HeaderSequenceEventArgs_hpp__pyplusplus_wrapper
#define HeaderSequenceEventArgs_hpp__pyplusplus_wrapper

void register_HeaderSequenceEventArgs_class();

#endif//HeaderSequenceEventArgs_hpp__pyplusplus_wrapper
