// This file has been generated by Py++.

#ifndef OpenGL3Shader_hpp__pyplusplus_wrapper
#define OpenGL3Shader_hpp__pyplusplus_wrapper

void register_OpenGL3Shader_class();

#endif//OpenGL3Shader_hpp__pyplusplus_wrapper
