// This file has been generated by Py++.

#ifndef PushButton_hpp__pyplusplus_wrapper
#define PushButton_hpp__pyplusplus_wrapper

void register_PushButton_class();

#endif//PushButton_hpp__pyplusplus_wrapper
