// This file has been generated by Py++.

#include "boost/python.hpp"
#include "indexing_suite/container_suite.hpp"
#include "indexing_suite/vector.hpp"
#include "generators/include/python_CEGUI.h"
#include "TargetTypeStack.pypp.hpp"

namespace bp = boost::python;

void register_TargetTypeStack_class(){

    { //::std::vector< CEGUI::String >
        typedef bp::class_< std::vector< CEGUI::String > > TargetTypeStack_exposer_t;
        TargetTypeStack_exposer_t TargetTypeStack_exposer = TargetTypeStack_exposer_t( "TargetTypeStack" );
        bp::scope TargetTypeStack_scope( TargetTypeStack_exposer );
        TargetTypeStack_exposer.def( bp::indexing::vector_suite< std::vector< CEGUI::String > >() );
    }

}
