// This file has been generated by Py++.

#ifndef TypedPropertyColourRect_hpp__pyplusplus_wrapper
#define TypedPropertyColourRect_hpp__pyplusplus_wrapper

void register_TypedPropertyColourRect_class();

#endif//TypedPropertyColourRect_hpp__pyplusplus_wrapper
