// This file has been generated by Py++.

#ifndef GUIContextRenderTargetEventArgs_hpp__pyplusplus_wrapper
#define GUIContextRenderTargetEventArgs_hpp__pyplusplus_wrapper

void register_GUIContextRenderTargetEventArgs_class();

#endif//GUIContextRenderTargetEventArgs_hpp__pyplusplus_wrapper
