// This file has been generated by Py++.

#ifndef SystemKeys_hpp__pyplusplus_wrapper
#define SystemKeys_hpp__pyplusplus_wrapper

void register_SystemKeys_class();

#endif//SystemKeys_hpp__pyplusplus_wrapper
