// This file has been generated by Py++.

#ifndef NamedElement_hpp__pyplusplus_wrapper
#define NamedElement_hpp__pyplusplus_wrapper

void register_NamedElement_class();

#endif//NamedElement_hpp__pyplusplus_wrapper
