// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "PyCEGUI_enumerations.pypp.hpp"

namespace bp = boost::python;

void register_enumerations(){

    bp::enum_< CEGUI::AspectMode>("AspectMode")
        .value("AM_IGNORE", CEGUI::AM_IGNORE)
        .value("AM_SHRINK", CEGUI::AM_SHRINK)
        .value("AM_EXPAND", CEGUI::AM_EXPAND)
        .export_values()
        ;

    bp::enum_< CEGUI::AutoScaledMode>("AutoScaledMode")
        .value("ASM_Disabled", CEGUI::ASM_Disabled)
        .value("ASM_Vertical", CEGUI::ASM_Vertical)
        .value("ASM_Horizontal", CEGUI::ASM_Horizontal)
        .value("ASM_Min", CEGUI::ASM_Min)
        .value("ASM_Max", CEGUI::ASM_Max)
        .value("ASM_Both", CEGUI::ASM_Both)
        .export_values()
        ;

    bp::enum_< CEGUI::BlendMode>("BlendMode")
        .value("BM_INVALID", CEGUI::BM_INVALID)
        .value("BM_NORMAL", CEGUI::BM_NORMAL)
        .value("BM_RTT_PREMULTIPLIED", CEGUI::BM_RTT_PREMULTIPLIED)
        .export_values()
        ;

    bp::enum_< CEGUI::ChildEventAction>("ChildEventAction")
        .value("CEA_REDRAW", CEGUI::CEA_REDRAW)
        .value("CEA_LAYOUT", CEGUI::CEA_LAYOUT)
        .export_values()
        ;

    bp::enum_< CEGUI::DimensionOperator>("DimensionOperator")
        .value("DOP_NOOP", CEGUI::DOP_NOOP)
        .value("DOP_ADD", CEGUI::DOP_ADD)
        .value("DOP_SUBTRACT", CEGUI::DOP_SUBTRACT)
        .value("DOP_MULTIPLY", CEGUI::DOP_MULTIPLY)
        .value("DOP_DIVIDE", CEGUI::DOP_DIVIDE)
        .export_values()
        ;

    bp::enum_< CEGUI::DimensionType>("DimensionType")
        .value("DT_LEFT_EDGE", CEGUI::DT_LEFT_EDGE)
        .value("DT_X_POSITION", CEGUI::DT_X_POSITION)
        .value("DT_TOP_EDGE", CEGUI::DT_TOP_EDGE)
        .value("DT_Y_POSITION", CEGUI::DT_Y_POSITION)
        .value("DT_RIGHT_EDGE", CEGUI::DT_RIGHT_EDGE)
        .value("DT_BOTTOM_EDGE", CEGUI::DT_BOTTOM_EDGE)
        .value("DT_WIDTH", CEGUI::DT_WIDTH)
        .value("DT_HEIGHT", CEGUI::DT_HEIGHT)
        .value("DT_X_OFFSET", CEGUI::DT_X_OFFSET)
        .value("DT_Y_OFFSET", CEGUI::DT_Y_OFFSET)
        .value("DT_INVALID", CEGUI::DT_INVALID)
        .export_values()
        ;

    bp::enum_< CEGUI::FontMetricType>("FontMetricType")
        .value("FMT_LINE_SPACING", CEGUI::FMT_LINE_SPACING)
        .value("FMT_BASELINE", CEGUI::FMT_BASELINE)
        .value("FMT_HORZ_EXTENT", CEGUI::FMT_HORZ_EXTENT)
        .export_values()
        ;

    bp::enum_< CEGUI::FrameImageComponent>("FrameImageComponent")
        .value("FIC_BACKGROUND", CEGUI::FIC_BACKGROUND)
        .value("FIC_TOP_LEFT_CORNER", CEGUI::FIC_TOP_LEFT_CORNER)
        .value("FIC_TOP_RIGHT_CORNER", CEGUI::FIC_TOP_RIGHT_CORNER)
        .value("FIC_BOTTOM_LEFT_CORNER", CEGUI::FIC_BOTTOM_LEFT_CORNER)
        .value("FIC_BOTTOM_RIGHT_CORNER", CEGUI::FIC_BOTTOM_RIGHT_CORNER)
        .value("FIC_LEFT_EDGE", CEGUI::FIC_LEFT_EDGE)
        .value("FIC_RIGHT_EDGE", CEGUI::FIC_RIGHT_EDGE)
        .value("FIC_TOP_EDGE", CEGUI::FIC_TOP_EDGE)
        .value("FIC_BOTTOM_EDGE", CEGUI::FIC_BOTTOM_EDGE)
        .value("FIC_FRAME_IMAGE_COUNT", CEGUI::FIC_FRAME_IMAGE_COUNT)
        .export_values()
        ;

    bp::enum_< CEGUI::HorizontalAlignment>("HorizontalAlignment")
        .value("HA_LEFT", CEGUI::HA_LEFT)
        .value("HA_CENTRE", CEGUI::HA_CENTRE)
        .value("HA_RIGHT", CEGUI::HA_RIGHT)
        .export_values()
        ;

    bp::enum_< CEGUI::HorizontalFormatting>("HorizontalFormatting")
        .value("HF_LEFT_ALIGNED", CEGUI::HF_LEFT_ALIGNED)
        .value("HF_CENTRE_ALIGNED", CEGUI::HF_CENTRE_ALIGNED)
        .value("HF_RIGHT_ALIGNED", CEGUI::HF_RIGHT_ALIGNED)
        .value("HF_STRETCHED", CEGUI::HF_STRETCHED)
        .value("HF_TILED", CEGUI::HF_TILED)
        .export_values()
        ;

    bp::enum_< CEGUI::HorizontalTextFormatting>("HorizontalTextFormatting")
        .value("HTF_LEFT_ALIGNED", CEGUI::HTF_LEFT_ALIGNED)
        .value("HTF_RIGHT_ALIGNED", CEGUI::HTF_RIGHT_ALIGNED)
        .value("HTF_CENTRE_ALIGNED", CEGUI::HTF_CENTRE_ALIGNED)
        .value("HTF_JUSTIFIED", CEGUI::HTF_JUSTIFIED)
        .value("HTF_WORDWRAP_LEFT_ALIGNED", CEGUI::HTF_WORDWRAP_LEFT_ALIGNED)
        .value("HTF_WORDWRAP_RIGHT_ALIGNED", CEGUI::HTF_WORDWRAP_RIGHT_ALIGNED)
        .value("HTF_WORDWRAP_CENTRE_ALIGNED", CEGUI::HTF_WORDWRAP_CENTRE_ALIGNED)
        .value("HTF_WORDWRAP_JUSTIFIED", CEGUI::HTF_WORDWRAP_JUSTIFIED)
        .export_values()
        ;

    bp::enum_< CEGUI::LoggingLevel>("LoggingLevel")
        .value("Errors", CEGUI::Errors)
        .value("Warnings", CEGUI::Warnings)
        .value("Standard", CEGUI::Standard)
        .value("Informative", CEGUI::Informative)
        .value("Insane", CEGUI::Insane)
        .export_values()
        ;

    bp::enum_< CEGUI::MouseButton>("MouseButton")
        .value("LeftButton", CEGUI::LeftButton)
        .value("RightButton", CEGUI::RightButton)
        .value("MiddleButton", CEGUI::MiddleButton)
        .value("X1Button", CEGUI::X1Button)
        .value("X2Button", CEGUI::X2Button)
        .value("MouseButtonCount", CEGUI::MouseButtonCount)
        .value("NoButton", CEGUI::NoButton)
        .export_values()
        ;

    bp::enum_< CEGUI::QuadSplitMode>("QuadSplitMode")
        .value("TopLeftToBottomRight", CEGUI::TopLeftToBottomRight)
        .value("BottomLeftToTopRight", CEGUI::BottomLeftToTopRight)
        .export_values()
        ;

    bp::enum_< CEGUI::RenderQueueID>("RenderQueueID")
        .value("RQ_USER_0", CEGUI::RQ_USER_0)
        .value("RQ_UNDERLAY", CEGUI::RQ_UNDERLAY)
        .value("RQ_USER_1", CEGUI::RQ_USER_1)
        .value("RQ_BASE", CEGUI::RQ_BASE)
        .value("RQ_USER_2", CEGUI::RQ_USER_2)
        .value("RQ_CONTENT_1", CEGUI::RQ_CONTENT_1)
        .value("RQ_USER_3", CEGUI::RQ_USER_3)
        .value("RQ_CONTENT_2", CEGUI::RQ_CONTENT_2)
        .value("RQ_USER_4", CEGUI::RQ_USER_4)
        .value("RQ_OVERLAY", CEGUI::RQ_OVERLAY)
        .value("RQ_USER_5", CEGUI::RQ_USER_5)
        .export_values()
        ;

    bp::enum_< CEGUI::VerticalAlignment>("VerticalAlignment")
        .value("VA_TOP", CEGUI::VA_TOP)
        .value("VA_CENTRE", CEGUI::VA_CENTRE)
        .value("VA_BOTTOM", CEGUI::VA_BOTTOM)
        .export_values()
        ;

    bp::enum_< CEGUI::VerticalFormatting>("VerticalFormatting")
        .value("VF_TOP_ALIGNED", CEGUI::VF_TOP_ALIGNED)
        .value("VF_CENTRE_ALIGNED", CEGUI::VF_CENTRE_ALIGNED)
        .value("VF_BOTTOM_ALIGNED", CEGUI::VF_BOTTOM_ALIGNED)
        .value("VF_STRETCHED", CEGUI::VF_STRETCHED)
        .value("VF_TILED", CEGUI::VF_TILED)
        .export_values()
        ;

    bp::enum_< CEGUI::VerticalTextFormatting>("VerticalTextFormatting")
        .value("VTF_TOP_ALIGNED", CEGUI::VTF_TOP_ALIGNED)
        .value("VTF_CENTRE_ALIGNED", CEGUI::VTF_CENTRE_ALIGNED)
        .value("VTF_BOTTOM_ALIGNED", CEGUI::VTF_BOTTOM_ALIGNED)
        .export_values()
        ;

    bp::enum_< CEGUI::WindowUpdateMode>("WindowUpdateMode")
        .value("WUM_ALWAYS", CEGUI::WUM_ALWAYS)
        .value("WUM_NEVER", CEGUI::WUM_NEVER)
        .value("WUM_VISIBLE", CEGUI::WUM_VISIBLE)
        .export_values()
        ;

    bp::enum_< CEGUI::XMLResourceExistsAction>("XMLResourceExistsAction")
        .value("XREA_RETURN", CEGUI::XREA_RETURN)
        .value("XREA_REPLACE", CEGUI::XREA_REPLACE)
        .value("XREA_THROW", CEGUI::XREA_THROW)
        .export_values()
        ;

}
