// This file has been generated by Py++.

#ifndef TypedPropertyUint_hpp__pyplusplus_wrapper
#define TypedPropertyUint_hpp__pyplusplus_wrapper

void register_TypedPropertyUint_class();

#endif//TypedPropertyUint_hpp__pyplusplus_wrapper
