// This file has been generated by Py++.

#ifndef PropertyHelper_hpp__pyplusplus_wrapper
#define PropertyHelper_hpp__pyplusplus_wrapper

void register_PropertyHelper_class();

#endif//PropertyHelper_hpp__pyplusplus_wrapper
