// This file has been generated by Py++.

#ifndef ImagerySectionMap_hpp__pyplusplus_wrapper
#define ImagerySectionMap_hpp__pyplusplus_wrapper

void register_ImagerySectionMap_class();

#endif//ImagerySectionMap_hpp__pyplusplus_wrapper
