// This file has been generated by Py++.

#ifndef OpenGLRendererBase_hpp__pyplusplus_wrapper
#define OpenGLRendererBase_hpp__pyplusplus_wrapper

void register_OpenGLRendererBase_class();

#endif//OpenGLRendererBase_hpp__pyplusplus_wrapper
