// This file has been generated by Py++.

#ifndef EventActionIterator_hpp__pyplusplus_wrapper
#define EventActionIterator_hpp__pyplusplus_wrapper

void register_EventActionIterator_class();

#endif//EventActionIterator_hpp__pyplusplus_wrapper
