// This file has been generated by Py++.

#ifndef ActivationEventArgs_hpp__pyplusplus_wrapper
#define ActivationEventArgs_hpp__pyplusplus_wrapper

void register_ActivationEventArgs_class();

#endif//ActivationEventArgs_hpp__pyplusplus_wrapper
