// This file has been generated by Py++.

#ifndef RenderedStringParser_hpp__pyplusplus_wrapper
#define RenderedStringParser_hpp__pyplusplus_wrapper

void register_RenderedStringParser_class();

#endif//RenderedStringParser_hpp__pyplusplus_wrapper
