// This file has been generated by Py++.

#ifndef RenderTarget_hpp__pyplusplus_wrapper
#define RenderTarget_hpp__pyplusplus_wrapper

void register_RenderTarget_class();

#endif//RenderTarget_hpp__pyplusplus_wrapper
