// This file has been generated by Py++.

#ifndef PropertyDefinitionColourUDim_hpp__pyplusplus_wrapper
#define PropertyDefinitionColourUDim_hpp__pyplusplus_wrapper

void register_PropertyDefinitionColourUDim_class();

#endif//PropertyDefinitionColourUDim_hpp__pyplusplus_wrapper
