// This file has been generated by Py++.

#ifndef OperatorDim_hpp__pyplusplus_wrapper
#define OperatorDim_hpp__pyplusplus_wrapper

void register_OperatorDim_class();

#endif//OperatorDim_hpp__pyplusplus_wrapper
