// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "DragDropEventArgs.pypp.hpp"

namespace bp = boost::python;

struct DragDropEventArgs_wrapper : CEGUI::DragDropEventArgs, bp::wrapper< CEGUI::DragDropEventArgs > {

    DragDropEventArgs_wrapper(CEGUI::DragDropEventArgs const & arg )
    : CEGUI::DragDropEventArgs( arg )
      , bp::wrapper< CEGUI::DragDropEventArgs >(){
        // copy constructor
        
    }

    DragDropEventArgs_wrapper(::CEGUI::Window * wnd )
    : CEGUI::DragDropEventArgs( boost::python::ptr(wnd) )
      , bp::wrapper< CEGUI::DragDropEventArgs >(){
        // constructor
    
    }

    static ::CEGUI::DragContainer * get_dragDropItem(CEGUI::DragDropEventArgs const & inst ){
        return inst.dragDropItem;
    }
    
    static void set_dragDropItem( CEGUI::DragDropEventArgs & inst, ::CEGUI::DragContainer * new_value ){ 
        inst.dragDropItem = new_value;
    }

};

void register_DragDropEventArgs_class(){

    { //::CEGUI::DragDropEventArgs
        typedef bp::class_< DragDropEventArgs_wrapper, bp::bases< CEGUI::WindowEventArgs > > DragDropEventArgs_exposer_t;
        DragDropEventArgs_exposer_t DragDropEventArgs_exposer = DragDropEventArgs_exposer_t( "DragDropEventArgs", "*!\n\
        \n\
           EventArgs based class used for certain dragdrop notifications\n\
        *\n", bp::init< CEGUI::Window * >(( bp::arg("wnd") )) );
        bp::scope DragDropEventArgs_scope( DragDropEventArgs_exposer );
        bp::implicitly_convertible< CEGUI::Window *, CEGUI::DragDropEventArgs >();
        DragDropEventArgs_exposer.add_property( "dragDropItem"
                    , bp::make_function( (::CEGUI::DragContainer * (*)( ::CEGUI::DragDropEventArgs const & ))(&DragDropEventArgs_wrapper::get_dragDropItem), bp::return_internal_reference< >() )
                    , bp::make_function( (void (*)( ::CEGUI::DragDropEventArgs &,::CEGUI::DragContainer * ))(&DragDropEventArgs_wrapper::set_dragDropItem), bp::with_custodian_and_ward_postcall< 1, 2 >() ) );
    }

}
