// This file has been generated by Py++.

#ifndef AnimationManager_hpp__pyplusplus_wrapper
#define AnimationManager_hpp__pyplusplus_wrapper

void register_AnimationManager_class();

#endif//AnimationManager_hpp__pyplusplus_wrapper
