// This file has been generated by Py++.

#ifndef Interpolator_hpp__pyplusplus_wrapper
#define Interpolator_hpp__pyplusplus_wrapper

void register_Interpolator_class();

#endif//Interpolator_hpp__pyplusplus_wrapper
