// This file has been generated by Py++.

#ifndef PropertyInitialiserIterator_hpp__pyplusplus_wrapper
#define PropertyInitialiserIterator_hpp__pyplusplus_wrapper

void register_PropertyInitialiserIterator_class();

#endif//PropertyInitialiserIterator_hpp__pyplusplus_wrapper
