// This file has been generated by Py++.

#ifndef FalagardPropertyBaseBool_hpp__pyplusplus_wrapper
#define FalagardPropertyBaseBool_hpp__pyplusplus_wrapper

void register_FalagardPropertyBaseBool_class();

#endif//FalagardPropertyBaseBool_hpp__pyplusplus_wrapper
