// This file has been generated by Py++.

#ifndef ProgressBar_hpp__pyplusplus_wrapper
#define ProgressBar_hpp__pyplusplus_wrapper

void register_ProgressBar_class();

#endif//ProgressBar_hpp__pyplusplus_wrapper
