// This file has been generated by Py++.

#include "boost/python.hpp"
#include "generators/include/python_CEGUI.h"
#include "RenderTargetEventArgs.pypp.hpp"

namespace bp = boost::python;

struct RenderTargetEventArgs_wrapper : CEGUI::RenderTargetEventArgs, bp::wrapper< CEGUI::RenderTargetEventArgs > {

    RenderTargetEventArgs_wrapper(CEGUI::RenderTargetEventArgs const & arg )
    : CEGUI::RenderTargetEventArgs( arg )
      , bp::wrapper< CEGUI::RenderTargetEventArgs >(){
        // copy constructor
        
    }

    RenderTargetEventArgs_wrapper(::CEGUI::RenderTarget * target )
    : CEGUI::RenderTargetEventArgs( boost::python::ptr(target) )
      , bp::wrapper< CEGUI::RenderTargetEventArgs >(){
        // constructor
    
    }

    static ::CEGUI::RenderTarget * get_target(CEGUI::RenderTargetEventArgs const & inst ){
        return inst.target;
    }
    
    static void set_target( CEGUI::RenderTargetEventArgs & inst, ::CEGUI::RenderTarget * new_value ){ 
        inst.target = new_value;
    }

};

void register_RenderTargetEventArgs_class(){

    { //::CEGUI::RenderTargetEventArgs
        typedef bp::class_< RenderTargetEventArgs_wrapper, bp::bases< CEGUI::EventArgs > > RenderTargetEventArgs_exposer_t;
        RenderTargetEventArgs_exposer_t RenderTargetEventArgs_exposer = RenderTargetEventArgs_exposer_t( "RenderTargetEventArgs", "! EventArgs class passed to subscribers of RenderTarget events.\n", bp::init< CEGUI::RenderTarget * >(( bp::arg("target") )) );
        bp::scope RenderTargetEventArgs_scope( RenderTargetEventArgs_exposer );
        bp::implicitly_convertible< CEGUI::RenderTarget *, CEGUI::RenderTargetEventArgs >();
        RenderTargetEventArgs_exposer.add_property( "target"
                    , bp::make_function( (::CEGUI::RenderTarget * (*)( ::CEGUI::RenderTargetEventArgs const & ))(&RenderTargetEventArgs_wrapper::get_target), bp::return_internal_reference< >() )
                    , bp::make_function( (void (*)( ::CEGUI::RenderTargetEventArgs &,::CEGUI::RenderTarget * ))(&RenderTargetEventArgs_wrapper::set_target), bp::with_custodian_and_ward_postcall< 1, 2 >() )
                    , "! pointer to the RenderTarget that triggered the event.\n" );
    }

}
