// This file has been generated by Py++.

#ifndef PyCEGUI_enumerations_hpp__pyplusplus_wrapper
#define PyCEGUI_enumerations_hpp__pyplusplus_wrapper

void register_enumerations();

#endif//PyCEGUI_enumerations_hpp__pyplusplus_wrapper
