// This file has been generated by Py++.

#ifndef ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda_hpp__pyplusplus_wrapper
#define ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda_hpp__pyplusplus_wrapper

void register_ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda_class();

#endif//ConstBaseIterator_9b6259ec5f5ed443accef2ab6f116fda_hpp__pyplusplus_wrapper
