// This file has been generated by Py++.

#ifndef WindowFactoryManager_hpp__pyplusplus_wrapper
#define WindowFactoryManager_hpp__pyplusplus_wrapper

void register_WindowFactoryManager_class();

#endif//WindowFactoryManager_hpp__pyplusplus_wrapper
