// This file has been generated by Py++.

#ifndef FrameComponentVector_hpp__pyplusplus_wrapper
#define FrameComponentVector_hpp__pyplusplus_wrapper

void register_FrameComponentVector_class();

#endif//FrameComponentVector_hpp__pyplusplus_wrapper
