// This file has been generated by Py++.

#ifndef RenderedStringTextComponent_hpp__pyplusplus_wrapper
#define RenderedStringTextComponent_hpp__pyplusplus_wrapper

void register_RenderedStringTextComponent_class();

#endif//RenderedStringTextComponent_hpp__pyplusplus_wrapper
