// This file has been generated by Py++.

#ifndef FrameWindow_hpp__pyplusplus_wrapper
#define FrameWindow_hpp__pyplusplus_wrapper

void register_FrameWindow_class();

#endif//FrameWindow_hpp__pyplusplus_wrapper
